function EnableBabble(enabled, updateCheckBox)
	BabbleOptions.Enabled = enabled;
	if updateCheckBox then
		StatusWindow:EnableBabbleCheckBox(enabled);
	end
	StatusButton:SetButtonState(enabled);
	if enabled then
		if Settings.EnableChatWindow and Settings.AutoHiddenChatWindow then
			ShowChatWindow(true, true);
		end
	else
		ChatWindow:Hide();
		StatusWindow.ColoursWindow:Hide();
		if Settings.ShowChatWindow then
			Settings.AutoHiddenChatWindow = true;
		end
		ShowChatWindow(false, true, true);
	end

	ShowBabbleStatus();
end

function EnableChatWindow(enabled)
	Settings.EnableChatWindow = enabled;
	if enabled and BabbleOptions.Enabled then
		ChatWindow:Show();
	else
		ChatWindow:Hide();
	end
	StatusButton:ShowChatButton(enabled);
end

function ShowChatWindow(visible, setCheckBox, autoHidden)
	if visible then
		ChatWindow:Show();
	else
		ChatWindow:Hide();
		if autoHidden == nil or autoHidden == false then
			Settings.AutoHiddenChatWindow = nil;
		end
	end

	Settings.ShowChatWindow = visible;
	if setCheckBox and StatusWindow.ShowChatWindowCheckbox:IsChecked() ~= visible then
		StatusWindow.ShowChatWindowCheckbox.ManualSet = true;
		StatusWindow.ShowChatWindowCheckbox:SetChecked(visible);
	end
end

function ShowBabbleStatus()
	if BabbleOptions.Enabled then
		Turbine.Shell.WriteLine(Strings["babble_status_on"]);
	else
		Turbine.Shell.WriteLine(Strings["babble_status_off"]);
	end
end

function ListBabbleFilters()
	for _, channelType in pairs(ChannelTypes) do
		local helper = BabbleHelpers[channelType];
		if helper:GetStatus() then
			Turbine.Shell.WriteLine("");
			Turbine.Shell.WriteLine(Strings["channel_header"] .. " " .. Strings["channel_" .. channelType]);
			BabbleHelpers[channelType]:ListFilters();
		end
	end
end

function CreateBabbleTabSettings(tabNumber, name, showAllFiltered)
	Settings.ChatTabs[tabNumber] =
	{
		Name = name,
		Filtered = {},
		Blocked = {},
		Other = {}
	}

	for _, channelType in ipairs(SortedChannels) do
		Settings.ChatTabs[tabNumber].Filtered[channelType] = showAllFiltered;
		Settings.ChatTabs[tabNumber].Blocked[channelType] = false;
	end
end

function GetBabbleSettingsLevel()
	if GlobalSettings.UseGlobalSettings then
		return Turbine.DataScope.Account;
	else
		return Turbine.DataScope.Character;
	end
end

function SaveBabbleSettings(showMessage)
	BabbleSave(Turbine.DataScope.Account, GlobalSettingsFileName, GlobalSettings);
	
	local settingsLevel = GetBabbleSettingsLevel();
	if showMessage then
		BabbleSave(settingsLevel, SettingsFileName, Settings,
			function(result, message)
				if result then
					ShowBabbleMessage(Strings["settings_saved"], 1);
				else
					ShowBabbleMessage(Strings["settings_notsaved"] .. " " .. message, 2);
				end
			end);
	else
		BabbleSave(settingsLevel, SettingsFileName, Settings);
	end
	
	if settingsLevel == Turbine.DataScope.Account then
		BabbleSave(Turbine.DataScope.Character, SettingsFileName, Settings);
	end
end

function ShowBabbleMessage(message, level)
	local display;
	if level == nil or level == 0 then
		display = "";
	elseif level == 1 then
		display = "<rgb=#00FF00>";
	elseif level == 2 then
		display = "<rgb=#FF0000>";
	end
	
	display = display .. message;
	
	if level ~= nil and level > 0 then
		display = display .. "</rgb>";
	end
	
	Turbine.Shell.WriteLine(display);
end