import "Simbo.Babble.DefaultFilters";

BabbleHelper = class();

local playerName = Turbine.Gameplay.LocalPlayer:GetInstance():GetName();
local playerNameLower = string.ToLower(playerName);
local meLower = string.ToLower(Strings["me"]);
local youLower = string.ToLower(Strings["you"]);

function BabbleHelper:Constructor(channelType, enabled, filterSet, nameList)
	self.ChannelType = channelType;
	self.Enabled = enabled;
	self.PatternList = {};
	self.ValidNames = {};

	self:SelectPatternList(filterSet);
	self:SelectNameList(nameList);
end

function BabbleHelper:GetStatus()
	return self.Enabled;
end

function BabbleHelper:UpdateStatus(enabled)
	self.Enabled = enabled;
end

function BabbleHelper:SelectPatternList(patternList)
	Settings.ChannelData[self.ChannelType].CurrentPatternList = patternList;
	
	self.PreparedFilters = { };
	
	if patternList ~= nil then
		if DefaultFilters.System[patternList] ~= nil then
			self.PatternList = DefaultFilters.System[patternList];
		else
			self.PatternList = Settings.PatternLists[patternList];
		end
	elseif self.ChannelType == ChannelTypes[Turbine.ChatType.Emote] then
		self.PatternList = DefaultFilters.Emotes[LANGUAGE];
	else
		self.PatternList = DefaultFilters.System["-1"];
	end

	local filterIndex = 1;
	local done = false;
	while not done do
		local stringIndex = tostring(filterIndex);
		local filterItem = self.PatternList[stringIndex];
		if filterItem ~= nil then
			local filter = filterItem.Text:ToLower();
			filter = filter:gsub("([%(%)%.%%%+%-%?%[%^%$%]])", "%%%1");
			filter = filter:gsub("*", ".+");
			filter = filter:gsub("_", "%%S+");
			if self.ChannelType == ChannelTypes[Turbine.ChatType.Emote] then
				if not filter:match(".+[%.!]$") then
					filter = filter .. "%s*[%.!]?";
				end
			end
			self.PreparedFilters[stringIndex] = "^" .. filter .. "$";
			filterIndex = filterIndex + 1;
		else
			done = true;
		end
	end
end

function BabbleHelper:SelectNameList(nameList)
	Settings.ChannelData[self.ChannelType].CurrentNameList = nameList;

	if nameList ~= nil and nameList ~= "0" then
		self.ValidNames = Settings.NameLists[Settings.ChannelData[self.ChannelType].CurrentNameList];
	else
		self.ValidNames = { };
	end
end

function BabbleHelper:ListFilters()
	local filterType = "";
	if self.PatternList.Type == "B" then
		filterType = Strings["block_patterns"];
	else
		filterType = Strings["allow_patterns"];
	end
	Turbine.Shell.WriteLine(Strings["filter_type"] .. ": " .. filterType);
	
	local filterIndex = 1;
	local done = false;
	while not done do
		local stringIndex = tostring(filterIndex);
		local filterItem = self.PatternList[stringIndex];
		if filterItem ~= nil then
			Turbine.Shell.WriteLine(" - " .. filterItem.Text);
			filterIndex = filterIndex + 1;
		else
			done = true;
		end
	end
end

function BabbleHelper:ProcessMessage(message)
	if not self.Enabled then
		ChatWindow:AddLine(message, self.ChannelType, "Filtered");
		return;
	end
	
	local checkMessage = message:ToLower():gsub("\194\160", " ");
	local channelData = Settings.ChannelData[self.ChannelType];
	
	local filterIndex = 1;
	local done = false;
	local matchesFilter = false;
	local isBlocked = false;
	local sourceName;
	local sourceMessage;
	
	if self.ChannelType == ChannelTypes[Turbine.ChatType.Say] then
		--Turbine.Shell.WriteLine(checkMessage:gsub("<", "["):gsub(">", "]"));
		if (checkMessage:match("<select:iid:0x030%w+>.*") or checkMessage:match("<select:iid:0x031%w+>.*")) and channelData.BlockNPC then
			matchesFilter = true;
			isBlocked = true;
		end 
	end
	
	local match1, match2 = checkMessage:match(NamePatterns[self.ChannelType][1]);
	if not match1 and NamePatterns[self.ChannelType][2] then
		match1, match2 = checkMessage:match(NamePatterns[self.ChannelType][2]);
	end
	
	if match1 then
		if match2 == nil then
			sourceName = youLower;
			sourceMessage = match1;
		else
			sourceName = match1;
			sourceMessage = match2;
		end
	else
		done = true;
	end

	while not done and not matchesFilter do
		local stringIndex = tostring(filterIndex);
		local filter = self.PreparedFilters[stringIndex];
		if filter ~= nil then
			if sourceMessage:match(filter) then
				local ignoreMatch = false;
				if self.PatternList[stringIndex].UnlessFrom and self.ValidNames ~= nil then
					ignoreMatch = IsInList(self.ValidNames, sourceName);
				end
				if not ignoreMatch and self.PatternList[stringIndex].UnlessTo then
					for _, name in pairs(self.ValidNames) do
						name = string.ToLower(name);
						if name == meLower then
							name = playerNameLower;
						end
						ignoreMatch = sourceMessage:match("%A" .. name .. "%A") or sourceMessage:match("%A" .. name .. "$");
						if ignoreMatch then
							break;
						end
					end
				end
				matchesFilter = not ignoreMatch;
			end
			filterIndex = filterIndex + 1;
		else
			done = true;
		end
	end
	
	if not isBlocked then
		if not match1 or (self.PatternList.Type == "B" and not matchesFilter) or (self.PatternList.Type == "A" and matchesFilter) then
			if not Settings.EnableChatWindow or channelData.RouteToStandard then
				local displayMessage = "<rgb=#" .. channelData.Colour .. ">" .. message .. "</rgb>";
				Turbine.Shell.WriteLine(displayMessage);
			end
		else
			isBlocked = true;
		end
	end
	
	local dataType = (isBlocked and "Blocked") or "Filtered";
	ChatWindow:AddLine(message, self.ChannelType, dataType);
end

function IsInList(items, value)
	local found = false;
	local lowerValue = string.ToLower(value);
	for _, item in pairs(items) do
		local lowerItem = string.ToLower(item);
		if lowerItem == meLower or lowerItem == playerNameLower then
			if lowerValue == playerNameLower or lowerValue == youLower then
				found = true;
				break;
			end
		elseif lowerItem == lowerValue then
			found = true;
			break;
		end
	end
	
	return found;
end
