import "Simbo.Babble.WordWrapper";
import "Simbo.Babble.List";

ChatList = class(Turbine.UI.Control);

local Constants =
{
	Border = 1,
	ScrollBarSize = 10,
	LineHeight = 14,
	Colour = Turbine.UI.Color(1.0, 0.0, 0.8, 0.0);
}

function ChatList:Constructor()
	Turbine.UI.Control.Constructor(self);
	
	self.Font = Globals.Fonts.Label;
	
	self.WordWrapper = WordWrapper(self);
	self.WordWrapper:SetFont(self.Font);
	self.WordWrapper:SetMarkupEnabled(true);
	
	self.Labels = List.new();
	self.AdjustHeight = true;
	
	self:SetBackColor(Globals.BorderColours.ListBox);
	
	self.ChatListBox = Turbine.UI.ListBox();
	self.ChatListBox:SetParent(self);
	self.ChatListBox:SetPosition(Constants.Border, Constants.Border);
	self.ChatListBox:SetBackColor(Globals.BackgroundColours.ListBox);
	
	self.LinesScrollBar = Turbine.UI.Lotro.ScrollBar();
	self.LinesScrollBar:SetOrientation(Turbine.UI.Orientation.Vertical);
	self.LinesScrollBar:SetParent(self.ChatListBox);
	self.LinesScrollBar:SetZOrder(999);
	self.ChatListBox:SetVerticalScrollBar(self.LinesScrollBar);
	
	self.SizeChanged = function(sender, args)
		if self:IsVisible() then
			self:LayoutControls();
		else
			self.AdjustHeight = true;
		end
	end
	
	self.VisibleChanged = function(sender, args)
		if self:IsVisible() and self.AdjustHeight then
			self:LayoutControls();
		end
	end
end

function ChatList:Destroy()
	self.ChatListBox:ClearItems();
	local label;
	repeat
		label = List.popleft(self.Labels);
	until label == nil;
end

function ChatList:LayoutControls()
	local chatBoxWidth = self:GetWidth() - (Constants.Border * 2);
	local chatBoxHeight = self:GetHeight() - (Constants.Border * 2);
	self.ChatListBox:SetSize(chatBoxWidth, chatBoxHeight);
	
	local itemCount = self.ChatListBox:GetItemCount();
	for index = 1, itemCount do
		local label = self.ChatListBox:GetItem(index);
		self:LayoutLabel(label, chatBoxWidth - Constants.ScrollBarSize);
	end
	
	self.LinesScrollBar:SetSize(Constants.ScrollBarSize, chatBoxHeight);
	self.LinesScrollBar:SetPosition(chatBoxWidth - Constants.ScrollBarSize,  0);
	
	self.AdjustHeight = false;
end

function ChatList:SetFont(font)
	self.Font = font;
	
	self.WordWrapper:SetFont(self.Font);
	local itemCount = self.ChatListBox:GetItemCount();
	for index = 1, itemCount do
		local label = self.ChatListBox:GetItem(index);
		self:SetLabelText(label, label.OriginalText);
	end
end

function ChatList:AddLine(lineText, colour)
	local label;
	
	if Settings.ShowTimestamp then
		local currentDate = Turbine.Engine.GetDate();
		lineText = string.format("[%d:%s] %s", currentDate.Hour, tostring(currentDate.Minute):PadLeft(2, "0"), lineText);
	end
	
	if colour ~= nil then
		lineText = "<rgb=#" .. colour .. ">" .. lineText .. "</rgb>";
	end
	
	if self.ChatListBox:GetItemCount() >= tonumber(Settings.LinesToSave) then
		label = self.ChatListBox:GetItem(1);
		self.ChatListBox:RemoveItem(label);
	else
		label = List.popleft(self.Labels);
		if label == nil then
			label = Turbine.UI.Label();
			label:SetParent(self.ChatListBox);
			label:SetSelectable(true);
			label:SetMarkupEnabled(true);
			label:SetForeColor(Constants.Colour);
		end
	end

	label.OriginalText = lineText;
	self:SetLabelText(label, lineText);
	self.ChatListBox:AddItem(label);
	
	self.ChatListBox:SetSelectedIndex(self.ChatListBox:GetItemCount());
end

function ChatList:SetLabelText(label, lineText)
	label:SetFont(self.Font);
	label:SetText(lineText);
	
	label.TextWidth = self.WordWrapper:GetTextLength(lineText);
	self:LayoutLabel(label);
end

function ChatList.HexStringToColour(hexColour)
	local r = tonumber(hexColour:sub(1, 2), 16) / 255;
	local g = tonumber(hexColour:sub(3, 4), 16) / 255;
	local b = tonumber(hexColour:sub(5, 6), 16) / 255;
	
	return Turbine.UI.Color(r, g, b);
end

function ChatList:ClearLines()
	local itemCount = self.ChatListBox:GetItemCount();
	for index = 1, itemCount do
		local label = self.ChatListBox:GetItem(1);
		self.ChatListBox:RemoveItem(label);
		List.pushright(self.Labels, label);
	end
end

function ChatList:LayoutLabel(label, labelWidth)
	if labelWidth == nil then
		labelWidth = self.ChatListBox:GetWidth() - Constants.ScrollBarSize;
	end
	self.WordWrapper:SetHeight(label, labelWidth, label.TextWidth, Constants.LineHeight, 1);
end