import "Simbo.Babble.WindowLotro";
import "Simbo.Babble.ChatList";
import "Simbo.Babble.ConfirmationWindow";

ChatWindow = class(WindowBasic);

local Constants =
{
	MinimumSize =
	{
		Width = 250,
		Height = 125
	},
	Padding =
	{
		Top = 0,
		Left = 0,
		Bottom = 1,
		Right = 1
	},
	Tabs =
	{
		Width = 100,
		SelectedHeight = 22,
		UnselectedHeight = 18,
		Spacing = 2,
		Font = Turbine.UI.Lotro.Font.TrajanPro15,
		SelectedColour = Turbine.UI.Color(1.0, 1.0, 1.0, 1.0),
		UnselectedColour = Turbine.UI.Color(1.0, 0.6, 0.6, 0.6),
		Maximum = 5
	},
	NewText =
	{
		Width = 20,
		Font = Turbine.UI.Lotro.Font.TrajanProBold22,
		Colour = Turbine.UI.Color(1.0, 0.0, 0.75, 0.0)
	},
	Resizer =
	{
		Width = 15,
		Height = 15,
		Icon = "Simbo/Babble/Icons/BasicDrag.tga"
	},
	LineSpacing = 4,
	ColumnSpacing = 8
}

function ChatWindow:Constructor()
	self.AlwaysVisible = true;
	WindowBasic.Constructor(self);
	
	self.CurrentTab = "1";
	self.NextAvailableTab = "2";
	self.ContextMenuTab = "";
	self.TabLabels = {};
	self.ChatLists = {};
	self.Opacity = tonumber(Settings.ChatWindowOpacity) / 100;
	
	self:SetPosition(tonumber(Settings.ChatWindow.Position.Left) or 100, tonumber(Settings.ChatWindow.Position.Top) or 100);
	self:SetSize(tonumber(Settings.ChatWindow.Size.Width) or Constants.MinimumSize.Width, tonumber(Settings.ChatWindow.Size.Height) or Constants.MinimumSize.Height);
	self:SetOpacity(self.Opacity);
	if self:GetWidth() < Constants.MinimumSize.Width then
		self:SetWidth(Constants.MinimumSize.Width);
	end
	if self:GetHeight() < Constants.MinimumSize.Height then
		self:SetHeight(Constants.MinimumSize.Height);
	end
	
	self.ConfirmWindow = ConfirmationWindow("Small");
	self.ConfirmWindow:SetZOrder(self:GetZOrder() + 10);
	self.ConfirmWindow.ButtonClick = function (sender,args)
		if args.Text:len() > 0 then
			Settings.ChatTabs[self.ConfirmWindow.TabNumber].Name = args.Text;
			self.TabLabels[self.ConfirmWindow.TabNumber]:SetText(args.Text);
		end
	end
	
	local top = Constants.Padding.Top;
	local left = Constants.Padding.Left;

	local tabIndex = 1;
	local tabNumber = tostring(tabIndex);
	while Settings.ChatTabs[tabNumber] ~= nil do
		local chatList = self:AddNewTabToWindow(tabNumber);
		
		local chatTab = Settings.ChatTabs[tabNumber];
		if chatTab.FontSize ~= nil then
			chatList:SetFont(Globals.Fonts["Chat" .. chatTab.FontSize]);
		end

		tabIndex = tabIndex + 1;
		tabNumber = tostring(tabIndex);
	end
	self.NextAvailableTab = tabNumber;
	
	self:CreateContextMenu();
	self:SelectTab(self.CurrentTab)
	
	self:SetResizer(Constants.MinimumSize.Width, Constants.MinimumSize.Height);
	if not self:AdjustResizer() then
		self:LayoutControls();
	end
end

function ChatWindow:UpdateWindowOpacity(opacity)
	self.Opacity = opacity;
	self:SetOpacity(opacity);
end

function ChatWindow:AdjustResizer()
	local totalTabs = tonumber(self.NextAvailableTab) - 1;
	local widthForTabs = totalTabs * (Constants.Tabs.Width + Constants.Tabs.Spacing);
	local minimumWidth = Constants.MinimumSize.Width;
	if widthForTabs > minimumWidth then
		minimumWidth = widthForTabs;
	end
	
	return self:UpdateResizer(minimumWidth, Constants.MinimumSize.Height);
end

function ChatWindow:LayoutControls()
	local tabIndex = 1;
	tabNumber = tostring(tabIndex);
	while self.ChatLists[tabNumber] ~= nil do
		self:SetChatListSize(self.ChatLists[tabNumber]);
		
		tabIndex = tabNumber + 1;
		tabNumber = tostring(tabIndex);
	end
end

function ChatWindow:SetChatListSize(chatList)
	chatList:SetSize(self:GetWidth() - Constants.Padding.Left - Constants.Padding.Right, self:GetHeight() - chatList:GetTop() - Constants.Padding.Bottom); 
end

function ChatWindow:Show()
	self:SetVisible(true);
	self:Activate();
end

function ChatWindow:Hide()
	self:Close();
end

function ChatWindow:SelectTab(selectTabNumber)
	self.CurrentTab = selectTabNumber;
	
	local currentTab = self.TabLabels[self.CurrentTab];
	if currentTab.HasNewText then
		currentTab.NewTextMarker:SetVisible(false);
		currentTab.HasNewText = false;
	end
	
	local tabIndex = 1;
	local tabNumber = tostring(tabIndex);
	while Settings.ChatTabs[tabNumber] ~= nil do
		local chatList = self.ChatLists[tabNumber];
		chatList:SetVisible(tabNumber == selectTabNumber);
		
		local tabLabel = self.TabLabels[tabNumber];
		self:StyleTabLabel(tabLabel, tabNumber == selectTabNumber);
		
		tabIndex = tabNumber + 1;
		tabNumber = tostring(tabIndex);
	end
end

function ChatWindow:StyleTabLabel(tabLabel, isSelected)
	if isSelected then
		tabLabel:SetTop(0);
		tabLabel:SetHeight(Constants.Tabs.SelectedHeight);
		tabLabel:SetForeColor(Constants.Tabs.SelectedColour);
	else
		local unselectedTop = (Constants.Tabs.SelectedHeight - Constants.Tabs.UnselectedHeight) / 2;
		
		tabLabel:SetTop(unselectedTop);
		tabLabel:SetHeight(Constants.Tabs.UnselectedHeight);
		tabLabel:SetForeColor(Constants.Tabs.UnselectedColour);
	end
end

function ChatWindow:ShowContextMenuItemForTab(tabNumber)
	self.ContextMenuTab = tabNumber;

	local tab = Settings.ChatTabs[self.ContextMenuTab];
	self:SetContextMenuItems(self.FilteredContextMenu:GetItems(), tab.Filtered);
	self:SetContextMenuItems(self.BlockedContextMenu:GetItems(), tab.Blocked);
	if tab.Other ~= nil then
		self:SetContextMenuItems(self.OtherContextMenu:GetItems(), tab.Other);
	end
	
	local deleteMenuItem = self.TabContextMenu:GetItems():Get(3);
	deleteMenuItem:SetEnabled(Settings.ChatTabs["2"] ~= nil);
	
	self:SetFontMenuItems();
	
	local totalTabs = tonumber(self.NextAvailableTab) - 1;
	local createMenuItem = self.TabContextMenu:GetItems():Get(1);
	createMenuItem:SetEnabled(totalTabs < Constants.Tabs.Maximum);

	self.TabContextMenu:ShowMenu();
end

function ChatWindow:SetContextMenuItems(menuItems, tabData)
	local itemCount = menuItems:GetCount();
	
	for itemIndex = 1, itemCount do
		local menuItem = menuItems:Get(itemIndex);
		menuItem:SetChecked(tabData[menuItem.ChannelType]);
	end
end

function ChatWindow:SetFontMenuItems()
	local menuItems = self.FontContextMenu:GetItems();
	local itemCount = menuItems:GetCount();
	local chatTabFontSize = Settings.ChatTabs[self.ContextMenuTab].FontSize or Globals.DefaultFontSize
	
	for itemIndex = 1, itemCount do
		local menuItem = menuItems:Get(itemIndex);
		menuItem:SetChecked(chatTabFontSize == menuItem.FontSize);
	end
end

function ChatWindow:AddLine(text, channelType, dataType)
	if not Settings.EnableChatWindow then
		return;
	end

	local tabIndex = 1;
	local tabNumber = tostring(tabIndex);
	while Settings.ChatTabs[tabNumber] ~= nil do
		local tabData = Settings.ChatTabs[tabNumber][dataType];
		if tabData ~= nil and tabData[channelType] then
			local channelData = Settings.ChannelData[channelType]
			self.ChatLists[tabNumber]:AddLine(text, channelData.Colour);
			local tabLabel = self.TabLabels[tabNumber]
			if tabNumber ~= self.CurrentTab and not tabLabel.HasNewText then
				tabLabel.HasNewText = true;
				tabLabel.NewTextMarker:SetVisible(true);
			end
		end
		
		tabIndex = tabIndex + 1;
		tabNumber = tostring(tabIndex);
	end
end

function ChatWindow:AddNewTabToWindow(tabNumber)
	local tabLabel = self:CreateTabLabel(Settings.ChatTabs[tabNumber].Name, tabNumber);
	self:StyleTabLabel(tabLabel, false);
	
	self.TabLabels[tabNumber] = tabLabel;
	
	local chatList = ChatList();
	chatList:SetParent(self);
	chatList:SetPosition(Constants.Padding.Left, Constants.Padding.Top + Constants.Tabs.SelectedHeight);
	chatList:SetVisible(false);
	
	self.ChatLists[tabNumber] = chatList;
	
	return chatList;
end

function ChatWindow:CreateTabLabel(text, tabNumber)
	local label = CreateLabel(self);
	label:SetWidth(Constants.Tabs.Width);
	label:SetOpacity(self.Opacity);
	label:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter);
	label:SetBackColor(Globals.BackgroundColours.ListBox);
	label:SetFont(Constants.Tabs.Font);
	label:SetText(text);
	self:SetTabLabelNumber(label, tabNumber);
	label.HasNewText = false;

	label.NewTextMarker = CreateLabel(self);
	label.NewTextMarker:SetWidth(Constants.NewText.Width);
	label.NewTextMarker:SetOpacity(self.Opacity);
	label.NewTextMarker:SetTextAlignment(Turbine.UI.ContentAlignment.TopRight);
	label.NewTextMarker:SetForeColor(Constants.NewText.Colour)
	label.NewTextMarker:SetFont(Constants.NewText.Font);
	label.NewTextMarker:SetText("*");
	label.NewTextMarker:SetVisible(false);
	
	label.PositionChanged = function(sender, args) 
		sender.NewTextMarker:SetPosition(sender:GetLeft() + sender:GetWidth() - sender.NewTextMarker:GetWidth() - 2, sender:GetTop());
	end
	
	label.MouseDown = function(sender, args)
		if args.Button == Turbine.UI.MouseButton.Left then
			sender.DragStartX = args.X;
			sender.DragStartY = args.Y;
			sender.Dragging = true;
			sender.Dragged = false;
			
			self:SelectTab(sender.TabNumber);
		end
	end
	
	label.MouseUp = function(sender, args) 
		if args.Button == Turbine.UI.MouseButton.Left then			
			sender.Dragging = false;
			--if not sender.Dragged then
			--	self:SelectTab(sender.TabNumber);
			--end
		elseif args.Button == Turbine.UI.MouseButton.Right then
			self:ShowContextMenuItemForTab(label.TabNumber);
		end
	end
	
	label.MouseMove = function(sender,args)
		if sender.Dragging then
			local left, top = self:GetPosition();
			self:SetPosition(left + (args.X - sender.DragStartX), top + args.Y - sender.DragStartY);
			sender.Dragged = true;
			if self:GetLeft() > Turbine.UI.Display.GetWidth() - self:GetWidth() then
				self:SetLeft(Turbine.UI.Display.GetWidth() - self:GetWidth());				
			end
			if self:GetLeft() < 0 then
				self:SetLeft(0);				
			end			
			if self:GetTop() > Turbine.UI.Display.GetHeight() - self:GetHeight() then
				self:SetTop(Turbine.UI.Display.GetHeight() - self:GetHeight());				
			end
			if self:GetTop() < 0 then
				self:SetTop(0);				
			end			
		end
	end
	
	return label;
end

function ChatWindow:SetTabLabelNumber(tabLabel, tabNumber)
	local tabIndex = tonumber(tabNumber) - 1;
	local left = Constants.Padding.Left + (Constants.Tabs.Width + Constants.Tabs.Spacing) * tabIndex;
	tabLabel:SetLeft(left);
	tabLabel.TabNumber = tabNumber;
end

function ChatWindow:CreateContextMenu()
	self.TabContextMenu = Turbine.UI.ContextMenu();
	
	local mainMenuItems = self.TabContextMenu:GetItems();
	
	self:CreateTabMenuItem(mainMenuItems, Strings["create_tab"], "CREATE");
	self:CreateTabMenuItem(mainMenuItems, Strings["rename_tab"], "RENAME");
	self:CreateTabMenuItem(mainMenuItems, Strings["delete_tab"], "DELETE");
	self.FontContextMenu = self:CreateFontMenu(mainMenuItems);
	self:CreateTabMenuItem(mainMenuItems, Strings["clear_tab"], "CLEAR");

	self.FilteredContextMenu = self:CreateSubMenu(mainMenuItems, "Filtered", Strings["filtered_menu"]);
	self.BlockedContextMenu = self:CreateSubMenu(mainMenuItems, "Blocked", Strings["blocked_menu"]);
	self.OtherContextMenu = self:CreateOtherSubMenu(mainMenuItems, Strings["other_menu"] );
end

function ChatWindow:CreateTabMenuItem(mainMenuItems, text, action)
	local menuItem = Turbine.UI.MenuItem(text);
	menuItem.Action = action;
    mainMenuItems:Add(menuItem);
	menuItem.Click = function(sender, args)
		if sender.Action == "CREATE" then
			self.ConfirmWindow:Hide();
			self:CreateTab();
		elseif sender.Action == "RENAME" then 
			self.ConfirmWindow.TabNumber = self.ContextMenuTab;
			self.ConfirmWindow:ShowWindow(Strings["rename_tab"], Strings["rename_tab_confirm"], true, false, Settings.ChatTabs[self.ContextMenuTab].Name);
		elseif sender.Action == "DELETE" then 
			self.ConfirmWindow:Hide();
			self:DeleteTab();
		elseif sender.Action == "CLEAR" then
			self.ChatLists[self.ContextMenuTab]:ClearLines();
		end
	end
end

function ChatWindow:CreateFontMenu(mainMenuItems)
	local subMenuItem = Turbine.UI.MenuItem(Strings["font_size"]);
	mainMenuItems:Add(subMenuItem);
	local subMenuItems = subMenuItem:GetItems();
	
	for fontSize = 12, 20, 2 do
		menuItem = Turbine.UI.MenuItem(Strings[fontSize .. "px"]);
		--menuItem:SetFont(Globals.Fonts["Chat" .. fontSize]);
		menuItem.FontSize = tostring(fontSize);
		subMenuItems:Add(menuItem);
		menuItem.Click = function(sender, args)
			Settings.ChatTabs[self.ContextMenuTab].FontSize = sender.FontSize;
			self.ChatLists[self.ContextMenuTab]:SetFont(Globals.Fonts["Chat" .. sender.FontSize]);
		end
	end
	
	return subMenuItem;
end

function ChatWindow:CreateSubMenu(mainMenuItems, subMenuType, subMenuHeader)
	local subMenuItem = Turbine.UI.MenuItem(subMenuHeader);
	mainMenuItems:Add(subMenuItem);
	local subMenuItems = subMenuItem:GetItems();
	
	for _, channelType in ipairs(SortedChannels) do
		menuItem = Turbine.UI.MenuItem(Strings["channel_" .. channelType]);
		menuItem.ChannelType = channelType;
		menuItem.MenuType = subMenuType;
		subMenuItems:Add(menuItem);
		menuItem.Click = function(sender, args)
			sender:SetChecked(not sender:IsChecked());
			Settings.ChatTabs[self.ContextMenuTab][sender.MenuType][sender.ChannelType] = sender:IsChecked();
			return nil;
		end
	end
	
	return subMenuItem;
end

function ChatWindow:CreateOtherSubMenu(mainMenuItems, subMenuHeader)
	local subMenuItem = Turbine.UI.MenuItem(subMenuHeader);
	mainMenuItems:Add(subMenuItem);
	local subMenuItems = subMenuItem:GetItems();
	
	for _, channelType in ipairs(SortedOtherChannels) do
		menuItem = Turbine.UI.MenuItem(Strings["channel_" .. channelType]);
		menuItem.ChannelType = channelType;
		menuItem.MenuType = subMenuType;
		subMenuItems:Add(menuItem);
		menuItem.Click = function(sender, args)
			sender:SetChecked(not sender:IsChecked());
			if Settings.ChatTabs[self.ContextMenuTab].Other == nil then
				Settings.ChatTabs[self.ContextMenuTab].Other = {};
			end
			Settings.ChatTabs[self.ContextMenuTab].Other[sender.ChannelType] = sender:IsChecked();
			return nil;
		end
	end
	
	return subMenuItem;
end

function ChatWindow:CreateTab()
	CreateBabbleTabSettings(self.NextAvailableTab, Strings["tab_name_new"], false);
	local chatList = self:AddNewTabToWindow(self.NextAvailableTab);
	self:SetChatListSize(chatList); 
	self.NextAvailableTab = tostring(tonumber(self.NextAvailableTab) + 1);
	self:AdjustResizer();
end

function ChatWindow:DeleteTab()
	local deleteTab = self.ContextMenuTab;
	local selectTab = nil;
	
	if self.CurrentTab == deleteTab then
		selectTab = "1";
	elseif tonumber(self.CurrentTab) > tonumber(deleteTab) then
		selectTab = tostring(tonumber(self.CurrentTab) - 1);
	end

	self.TabLabels[deleteTab]:SetParent(nil);
	self.ChatLists[deleteTab]:SetParent(nil);
	self.ChatLists[deleteTab]:Destroy();
		
	local nextTab = tostring(tonumber(deleteTab) + 1);
	while Settings.ChatTabs[nextTab] ~= nil do
		Settings.ChatTabs[deleteTab] = Settings.ChatTabs[nextTab];
		self.TabLabels[deleteTab] = self.TabLabels[nextTab];
		self.ChatLists[deleteTab] = self.ChatLists[nextTab];
		self:SetTabLabelNumber(self.TabLabels[deleteTab], deleteTab);
		
		deleteTab = nextTab;
		nextTab = tostring(tonumber(deleteTab) + 1);
	end
	
	Settings.ChatTabs[deleteTab] = nil;
	self.TabLabels[deleteTab] = nil;
	self.ChatLists[deleteTab] = nil;

	self.NextAvailableTab = tostring(tonumber(self.NextAvailableTab) - 1);
	self:AdjustResizer();
	
	if selectTab ~= nil then
		self:SelectTab(selectTab);
	end
end

function ChatWindow:SavePositionSettings()
	Settings.ChatWindow.Position.Top = tostring(self:GetTop());
	Settings.ChatWindow.Position.Left = tostring(self:GetLeft());
	Settings.ChatWindow.Size.Width = tostring(self:GetWidth());
	Settings.ChatWindow.Size.Height = tostring(self:GetHeight());
end