ColourPicker = class();

local Constants =
{
	ColourLabel =
	{
		Width = 75,
	},
	ColourChart =
	{
		GridWidth = 51,
		GridHeight = 15,
		GridSize = 3,
		Image = "Simbo/Babble/Icons/ColourPicker.tga"
	},
	ColumnSpacing = 8
}

local ColourPickerImage = nil;

function ColourPicker:Constructor(parent, hasLabel)
	local colourBoxHeight = Constants.ColourChart.GridHeight * Constants.ColourChart.GridSize;
	local colourBoxWidth = Constants.ColourChart.GridWidth * Constants.ColourChart.GridSize;
	
	self.DisplayItem = Turbine.UI.Control();
	self.DisplayItem:SetParent(parent);
	self.DisplayItem:SetMouseVisible(false);
	self.DisplayItem:SetSize(colourBoxWidth + Constants.ColumnSpacing + Constants.ColourLabel.Width, colourBoxHeight);
	
	if hasLabel == nil then	
		hasLabel = true
	end
	
	self.SelectedColourLabel = CreateLabel(self.DisplayItem, false);
	self.SelectedColourLabel:SetPosition(0, 0);
	self.SelectedColourLabel:SetWidth(Constants.ColourLabel.Width);
	self.SelectedColourLabel:SetText(Strings["pattern_colour"]);
	self.SelectedColourLabel:SetVisible(hasLabel);

	self.ColourChart = Turbine.UI.Control();
	self.ColourChart:SetParent(self.DisplayItem);
	if hasLabel then
		self.ColourChart:SetPosition(self.SelectedColourLabel:GetWidth() + Constants.ColumnSpacing, 0);
	else
		self.ColourChart:SetPosition(0, 0);
	end
	self.ColourChart:SetSize(colourBoxWidth, colourBoxHeight);
	if ColourPickerImage == nil then
		self.ColourChart:SetBackground(Constants.ColourChart.Image);
		ColourPickerImage = self.ColourChart:GetBackground();
	else
		self.ColourChart:SetBackground(ColourPickerImage);
	end
	self.ColourChart.MouseClick = function(sender, args)
		if args.Button == Turbine.UI.MouseButton.Left then
			local colour;
			local gridX = math.floor(args.X / Constants.ColourChart.GridSize);
			local gridY = math.floor(args.Y / Constants.ColourChart.GridSize);
			local l = ((Constants.ColourChart.GridHeight - 1) - gridY) / (Constants.ColourChart.GridHeight - 1)
			if gridX == 0 then
				colour = Turbine.UI.Color(l, l, l);
			else
				local h = (gridX - 1) * 2 / 100.0;
				colour = HslToRgb(h, 1, l);
			end
			self:SetSelectedColour(colour);
			
			self.ColourSelected(self, { Colour = colour });
		end
	end
	
	self.ColourSelected = function () end;
end

function ColourPicker:SetPosition(left, top)
	self.DisplayItem:SetPosition(left, top);
end

function ColourPicker:SetText(text)
	self.SelectedColourLabel:SetText(text);
end

function ColourPicker:SetVisible(visible)
	self.DisplayItem:SetVisible(visible);
end

function ColourPicker:GetHeight()
	return self.ColourChart:GetHeight();
end

function ColourPicker:SetSelectedColour(colour)
	self.SelectedColourLabel:SetForeColor(colour);
end

function ColourPicker:SetSelectedColourInHex(hexColour)
	local r = tonumber(hexColour:sub(1, 2), 16) / 255;
	local g = tonumber(hexColour:sub(3, 4), 16) / 255;
	local b = tonumber(hexColour:sub(5, 6), 16) / 255;
	
	self:SetSelectedColour(Turbine.UI.Color(r, g, b));
end

function ColourPicker:GetSelectedColour()
	return self.SelectedColourLabel:GetForeColor();
end

function ColourPicker:GetSelectedColourInHex()
	local colour = self:GetSelectedColour();
	local hexR = string.format("%x", colour.R * 255);
	local hexG = string.format("%x", colour.G * 255);
	local hexB = string.format("%x", colour.B * 255);

	return hexR:PadLeft(2, "0") .. hexG:PadLeft(2, "0") .. hexB:PadLeft(2, "0");
end

function HslToRgb(h, s, l)
	local r, g, b;

	if s == 0.0 then
		r = l;
		g = l;
		b = l;
	else
		local q, p;
		if l < 0.5 then
			q = l * (1.0 + s);
		else
			q = l + s - l * s;
		end

		p = 2.0 * l - q;
		
		r = HueToRgb(p, q, h + 1.0 / 3.0);
		g = HueToRgb(p, q, h);
		b = HueToRgb(p, q, h - 1.0 / 3.0);
	end

	return Turbine.UI.Color(r, g, b);
end

function HueToRgb(p, q, t)
	if t < 0.0 then
		t = t + 1.0;
	end
	
	if t > 1.0 then
		t = t - 1.0;
	end
	
	if t < 1.0 / 6.0 then
		return p + (q - p) * 6.0 * t;
	end
	
	if t < 1.0 / 2.0 then
		return q;
	end
	
	if t < 2.0 / 3.0 then
		return p + (q - p) * (2.0 / 3.0 - t) * 6.0;
	end
	
	return p;
end