import "Simbo.Babble.WindowLotro";
import "Simbo.Babble.ColourPicker";
import "Simbo.Babble.StyledList";

ColoursWindow = class(WindowLotro);

local Constants =
{
	MinimumSize =
	{
		Width = 315,
		Height = 280
	},
	Padding =
	{
		Top = 50,
		Left = 15,
		Bottom = 20,
		Right = 15
	},
	Sizes =
	{
		Label =
		{
			Width = 60,
		},
		NarrowLabel = 
		{
			Width = 12
		},
		TextBox = 
		{
			Width = 36
		},
		ListBox = 
		{
			Width = 110,
			Height = 182,
			ReducedHeight = 120
		}
	},
	LineSpacing = 6,
	ColumnSpacing = 6
}

local channelColours = {};

function ColoursWindow:Constructor()
	WindowLotro.Constructor(self);
	
	--self.ColourPickers = {};
	self.FullMode = true;

	self:SetText(Strings["edit_colours"]);
	self:SetPosition(tonumber(Settings.ColoursWindow.Position.Left) or 100, tonumber(Settings.ColoursWindow.Position.Top) or 100);
	self:SetSize(Constants.MinimumSize.Width, Constants.MinimumSize.Height);
	self:SetOpacity(Globals.Opacity);
	if self:GetWidth() < Constants.MinimumSize.Width then
		self:SetWidth(Constants.MinimumSize.Width);
	end
	if self:GetHeight() < Constants.MinimumSize.Height then
		self:SetHeight(Constants.MinimumSize.Height);
	end
	
	local top = Constants.Padding.Top;
	local left = Constants.Padding.Left;
	
	self.ColoursList = StyledList(self, 1, true);
	self.ColoursList:SetPosition(left, top);
	self.ColoursList:SetSize(Constants.Sizes.ListBox.Width, Constants.Sizes.ListBox.Height);
	self.ColoursList:SetBorderColor(Globals.BorderColours.ListBox);
	self.ColoursList:SetBackColor(Globals.BackgroundColours.ListBox);
	self.ColoursList:SetForeColor(Globals.Colours.ListBox);
	self.ColoursList:SetFont(Globals.Fonts.ListBox);
	self.ColoursList:SetOpacity(Globals.Opacity);
	self.ColoursList.SelectedIndexChanged = function(sender, args)
		self:SelectColour(args.SelectedIndex);
	end
	
	left = left + self.ColoursList:GetWidth() + Constants.ColumnSpacing;
	
	local channelHeader = CreateLabel(self);
	channelHeader:SetPosition(left, top);
	channelHeader:SetWidth(Constants.Sizes.Label.Width);
	channelHeader:SetText(Strings["channel_header"]);
	
	left = left + channelHeader:GetWidth();
	
	self.ChannelName = CreateLabel(self);
	self.ChannelName:SetPosition(left, top);
	self.ChannelName:SetWidth(self:GetWidth() - left - Constants.Padding.Right);
	
	top = top + channelHeader:GetHeight() + Constants.LineSpacing;
	left = channelHeader:GetLeft();
	
	local selectHeader = CreateLabel(self);
	selectHeader:SetPosition(left, top);
	selectHeader:SetWidth(self:GetWidth() - left - Constants.Padding.Right);
	selectHeader:SetText(Strings["select_colour"]);
	
	top = top + selectHeader:GetHeight() + Constants.LineSpacing;
	
	self.ColourPicker = ColourPicker(self, false);
	self.ColourPicker:SetPosition(left, top);
	self.ColourPicker.ColourSelected = function (sender, args)
		self:UpdateSelectedColour(args.Colour, true);
		self.ColoursList:Focus();
		self:SetSaveButtonsStatus(true);
	end
	
	top = top + self.ColourPicker:GetHeight() + Constants.LineSpacing;
	
	local rgbHeader = CreateLabel(self);
	rgbHeader:SetPosition(left, top);
	rgbHeader:SetWidth(self:GetWidth() - left - Constants.Padding.Right);
	rgbHeader:SetText(Strings["enter_rgb"]);
	
	top = top + rgbHeader:GetHeight() + Constants.LineSpacing;
	
	self.RBox = self:CreateColourTextBox(left, top, Strings["r"]);

	left = self.RBox:GetLeft() + self.RBox:GetWidth() + Constants.ColumnSpacing;
	self.GBox = self:CreateColourTextBox(left, top, Strings["g"]);

	left = self.GBox:GetLeft() + self.GBox:GetWidth() + Constants.ColumnSpacing;
	self.BBox = self:CreateColourTextBox(left, top, Strings["b"]);
	
	top = self:GetHeight() - Constants.Padding.Bottom - Globals.Sizes.Button.Height;
	left = Constants.Padding.Left;

	local buttonsWidth = Globals.Sizes.Button.Width * 2 + Constants.ColumnSpacing ;

	self.CancelButton = CreateButton(self);
	self.CancelButton:SetPosition((self:GetWidth() - buttonsWidth) / 2, top);
	self.CancelButton:SetText(Strings["cancel"]);
	self.CancelButton.MouseClick = function(sender, args)
		self:Hide();
	end
	
	self.SaveButton = CreateButton(self);
	self.SaveButton:SetPosition(self.CancelButton:GetLeft() + self.CancelButton:GetWidth() + Constants.ColumnSpacing, top);
	self.SaveButton:SetText(Strings["ok"]);
	self.SaveButton.MouseClick = function(sender, args)
		if args.Button == Turbine.UI.MouseButton.Left then
			self:SaveSettings(true)
		end
	end
	
	self.VisibleChanged = function(sender, args)
		if self:IsVisible() then
			self:CheckWindowPosition();
		end
	end
	
	self.SizeChanged = function(sender, args)
		self:LayoutControls();
	end

	self.Saved = function() end;
	
	self:AddChannelsToList(SortedChannels);
	self:AddChannelsToList(SortedOtherChannels);
	
	self:LayoutControls();
	--self:SetResizer(Constants.MinimumSize.Width, Constants.MinimumSize.Height);
end

function ColoursWindow:LayoutControls()
	
end

function ColoursWindow:CreateColourTextBox(left, top, text)
	local label = CreateLabel(self);
	label:SetPosition(left, top);
	label:SetWidth(Constants.Sizes.NarrowLabel.Width);
	label:SetText(text);
	
	left = left + label:GetWidth();
	
	local box = CreateTextBox(self);
	box:SetPosition(left, top);
	box:SetWidth(Constants.Sizes.TextBox.Width);
	
	box.FocusGained = function(sender, args)
		box.Value = box:GetText();
		box.OrignalValue = box:GetText();
		box:SetWantsUpdates(true);
	end
	
	box.Update = function(sender, args)
		local newValue = box:GetText();
		if newValue ~= box.Value then
			if math:IsValidInt(newValue, 0, 255) then
				box.Invalid = nil;
				self:UpdateColourFromBoxes();
			else
				box.Invalid = true;
			end
			box.Value = newValue;
			self:SetSaveButtonsStatus(true);
		end
	end
	
	box.FocusLost = function(sender, args)
		if box.Invalid then
			box:SetText(box.OrignalValue);
			self:UpdateColourFromBoxes();
			box.Invalid = nil;
		end
		box.Value = nil;
		box.OrignalValue = nil;
		box:SetWantsUpdates(false);
	end
	
	return box;
end

function ColoursWindow:SetMode(full)
	if self.FullMode ~= full then
		self.FullMode = full;
		
		if self.FullMode then
			self.ColoursList:SetHeight(Constants.Sizes.ListBox.Height);
		else
			self.ColoursList:SetHeight(Constants.Sizes.ListBox.ReducedHeight);
			if self.ColoursList:GetSelectedLineIndex() ~= nil and self.ColoursList:GetSelectedLineIndex() > table.getn(SortedChannels) then
				self.ColoursList:SelectLine(1);
				self:SelectColour(1);
			end
		end
	end
end

function ColoursWindow:Show(channelType)
	self:PopulateSettings();
	self:SetVisible(true);
	self:Activate();
end

function ColoursWindow:Hide()
	self:Close();
end

function ColoursWindow:SelectColour(index)
	local channel;
	local sortedCount = table.getn(SortedChannels);

	if index <= sortedCount then
		channel = SortedChannels[index];
	else
		channel = SortedOtherChannels[index - sortedCount];
	end
	
	self.ChannelName:SetText(Strings["channel_" .. channel]);
	self.ChannelName:SetForeColor(channelColours[channel]);
	self:UpdateRGBBoxes(channelColours[channel]);
end

function ColoursWindow:UpdateColourFromBoxes()
	local r, g, b;
	if self.RBox.Invalid == nil then
		r = tonumber(self.RBox:GetText()) / 255
	end
	if self.GBox.Invalid == nil then
		g = tonumber(self.GBox:GetText()) / 255
	end
	if self.BBox.Invalid == nil then
		b = tonumber(self.BBox:GetText()) / 255
	end
	
	if r ~= nil and g ~= nil and b ~= nil then
		self:UpdateSelectedColour(Turbine.UI.Color(r, g, b), false);
	end
end

function ColoursWindow:UpdateSelectedColour(colour, updateRBGBoxes)
	local selectedIndex = self.ColoursList:GetSelectedLineIndex();
	local channelType = self.ColoursList:GetSelectedValue();
	
	self.ChannelName:SetForeColor(colour);
	self.ColoursList:SetLineColour(selectedIndex, colour);
	if updateRBGBoxes then
		self:UpdateRGBBoxes(colour);
	end
	channelColours[channelType] = colour;
end

function ColoursWindow:UpdateRGBBoxes(colour)
	local r, g, b = colour.R * 255, colour.G * 255, colour.B * 255;
	self.RBox:SetText(tostring(math:RoundToInt(r)));
	self.GBox:SetText(tostring(math:RoundToInt(g)));
	self.BBox:SetText(tostring(math:RoundToInt(b)));
end

function ColoursWindow:AddChannelsToList(channels)
	for _, channelType in ipairs(channels) do
		self.ColoursList:AddItem(Strings["channel_" .. channelType], false, nil, channelType);
	end
end

function ColoursWindow:PopulateSettings()
	local index = self:PopulateChannelSettings(1, SortedChannels);
	self:PopulateChannelSettings(index, SortedOtherChannels);
	
	self.ColoursList:SelectLine(1);
	self:SelectColour(1);
	
	self:ResetSettings();
end

function ColoursWindow:PopulateChannelSettings(index, channels)
	for _, channelType in ipairs(channels) do
		local colour = self:GetColourFromHex(Settings.ChannelData[channelType].Colour);
		self.ColoursList:SetLineColour(index, colour);
		channelColours[channelType] = colour
		index = index + 1;
	end
	
	return index;
end

function ColoursWindow:ResetSettings()
	self:SetSaveButtonsStatus(false);
end

function ColoursWindow:SetSaveButtonsStatus(enabled)
	self.SaveButton:SetEnabled(enabled);
end

function ColoursWindow:SaveSettings(closeWindow)
	local index = 1;
	for _, channelType in ipairs(SortedChannels) do
		Settings.ChannelData[channelType].Colour = self:GetHexFromColour(channelColours[channelType]);
		index = index + 1;
	end
	for _, channelType in ipairs(SortedOtherChannels) do
		Settings.ChannelData[channelType].Colour = self:GetHexFromColour(channelColours[channelType]);
		index = index + 1;
	end
	
	self.Saved(self, {});
	
	if closeWindow then
		self:Hide();
	else
		self:ResetSettings();
	end
	
	return true;
end

function ColoursWindow:HasChanges()
	return self.ColourChanged;
end

function ColoursWindow:SavePositionSettings()
	Settings.ColoursWindow.Position.Top = tostring(self:GetTop());
	Settings.ColoursWindow.Position.Left = tostring(self:GetLeft());
end

function ColoursWindow:GetColourFromHex(hexColour)
	local r = tonumber(hexColour:sub(1, 2), 16) / 255;
	local g = tonumber(hexColour:sub(3, 4), 16) / 255;
	local b = tonumber(hexColour:sub(5, 6), 16) / 255;
	
	return Turbine.UI.Color(r, g, b);
end

function ColoursWindow:GetHexFromColour(colour)
	local hexR = string.format("%x", colour.R * 255);
	local hexG = string.format("%x", colour.G * 255);
	local hexB = string.format("%x", colour.B * 255);

	return hexR:PadLeft(2, "0") .. hexG:PadLeft(2, "0") .. hexB:PadLeft(2, "0");
end