import "Turbine.UI";
import "Turbine.UI.Lotro";
import "Turbine.Gameplay"
import "Simbo.Babble.Localisation";
import "Simbo.Babble.Class"; -- Turbine library included so that there's no outside dependencies
import "Simbo.Babble.Events";
import "Simbo.Babble.StringFunctions";
import "Simbo.Babble.NumberFunctions";
import "Simbo.Babble.WindowFunctions";
import "Simbo.Babble.BabbleFunctions";
import "Simbo.Babble.BabbleHelper";
import "Simbo.Babble.OptionsPanel";
import "Simbo.Babble.StatusButton";
import "Simbo.Babble.StatusWindow";
import "Simbo.Babble.ChatWindow";

ChannelTypes = {};
OtherChannelTypes = {};
SortedChannels = {};
SortedOtherChannels = {};

ChannelTypes[Turbine.ChatType.Emote] = "Emote";
ChannelTypes[Turbine.ChatType.Say] = "Say";
ChannelTypes[Turbine.ChatType.Raid] = "Raid";
ChannelTypes[Turbine.ChatType.Regional] = "Regional";
ChannelTypes[Turbine.ChatType.World] = "World";
ChannelTypes[Turbine.ChatType.OOC] = "OOC";

OtherChannelTypes[Turbine.ChatType.Fellowship] = "Fellowship";
OtherChannelTypes[Turbine.ChatType.Kinship] = "Kinship";
OtherChannelTypes[Turbine.ChatType.Officer] = "Officer";

for _, channelType in pairs(ChannelTypes) do
	table.insert(SortedChannels, channelType);
end
table.sort(SortedChannels, function (a, b) 
	return Strings["channel_" .. a] < Strings["channel_" .. b]; 
end);
	
for _, channelType in pairs(OtherChannelTypes) do
	table.insert(SortedOtherChannels, channelType);
end
table.sort(SortedOtherChannels, function (a, b) 
	return Strings["channel_" .. a] < Strings["channel_" .. b]; 
end);

BabbleOptions = { Enabled = true, DefaultColour = "808080" };

GlobalSettings =
{
	UseGlobalSettings = false
}

Settings = 
{
	PatternListWindow = 
	{
		Position = {},
		Size = {}
	},
	NameListWindow = 
	{
		Position = {},
		Size = {}
	},
	SettingsWindow = 
	{
		Position = {},
		Size = {}
	},
	ColoursWindow =
	{
		Position = {}
	},
	StatusWindow = 
	{
		Position = {}
	},
	ChatWindow = 
	{
		Position = {},
		Size = {}
	},
	StatusButton =
	{
		Position = {}
	},
	PatternLists = nil,
	NextPatternListId = "1",
	NameLists = nil,
	NextNameListId = "1",
	LinesToSave = "100",
	ChannelData = {},
	EnableChatWindow = false,
	ShowChatWindow = true,
	ChatWindowOpacity = "96",
	ShowTimestamp = false
}

Globals =
{
	Sizes =
	{
		Label = { Height = 20 },
		TextBox = { Width = 200, Height = 22 },
		Button = { Width = 90, Height = 24 },
		WideButton = { Width = 120, Height = 24 },
		ExtraWideButton = { Width = 160, Height = 24 },
		NarrowButton = { Width = 55, Height = 24 },
		CheckBoxOnly = { Width = 20, Height = 22 }
	},
	Fonts = 
	{
		Label = Turbine.UI.Lotro.Font.Verdana14,
		SmallLabel = Turbine.UI.Lotro.Font.Verdana12, 
		TextBox = Turbine.UI.Lotro.Font.Verdana14,
		ListBox = Turbine.UI.Lotro.Font.Verdana14,
		Chat12 = Turbine.UI.Lotro.Font.Verdana12,
		Chat14 = Turbine.UI.Lotro.Font.Verdana14,
		Chat16 = Turbine.UI.Lotro.Font.Verdana16,
		Chat18 = Turbine.UI.Lotro.Font.Verdana18,
		Chat20 = Turbine.UI.Lotro.Font.Verdana20
	},
	Colours =
	{
		Label = Turbine.UI.Color(1.0, 1.0, 1.0, 1.0),
		InactiveLabel = Turbine.UI.Color(1.0, 0.5, 0.5, 0.5),
		TextBox = Turbine.UI.Color(1.0, 1.0, 1.0, 1.0),
		ListBox = Turbine.UI.Color(1.0, 1.0, 1.0, 1.0),
		Confirmation = Turbine.UI.Color((210 / 255), 1, 0),
		Warning = Turbine.UI.Color(1.0, 1.0, 0.0, 0.0)
	},
	BackgroundColours =
	{
		TextBox = Turbine.UI.Color(1.0, 0, 0, 0),
		ListBox = Turbine.UI.Color(1.0, 0, 0, 0)
	},
	BorderColours =
	{
		ListBox = Turbine.UI.Color(1.0, 0.16, 0.19, 0.24)
	},
	Opacity = 0.96,
	DefaultFontSize = "14"
}

BabbleLoad = Turbine.PluginData.Load;
BabbleSave = Turbine.PluginData.Save;

GlobalSettingsFileName = "Babble2GlobalSettings";
GlobalSettings = BabbleLoad(Turbine.DataScope.Account, GlobalSettingsFileName) or GlobalSettings;

SettingsFileName = "Babble2Settings";
Settings = BabbleLoad(GetBabbleSettingsLevel(), SettingsFileName) or BabbleLoad(Turbine.DataScope.Character, "BabbleSettings") or Settings;

-- Upgrade Settings
if Settings.ChannelData == nil then
	Settings.ChannelData =
	{
		Emote =
		{
			Enabled = true,
			Colour = Settings.EmoteColour,
			CurrentPatternList = Settings.CurrentEmoteList or "1",
			CurrentNameList = Settings.CurrentNameList,
			RouteToStandard = true
		}
	};
	Settings.PatternListWindow = Settings.EmoteListWindow;
	Settings.PatternLists = Settings.EmoteLists;
	Settings.NextPatternListId = Settings.NextEmoteListId;
	for _, list in pairs(Settings.PatternLists) do
		list.Type = "B";
	end

	Settings.EmoteColour = nil;
	Settings.CurrentEmoteList = nil;
	Settings.CurrentNameList = nil;
	Settings.EmoteSetWindow = nil;
	Settings.EmoteListWindow = nil;
	Settings.EmoteLists = nil;
	Settings.NextEmoteListId = nil;
end

-- Other default settings
if Settings.PatternLists == nil or Settings.PatternLists["1"] == nil then
	Settings.NextPatternListId = "2";
	Settings.PatternLists =
	{
		["1"] = DefaultFilters.Emotes[LANGUAGE];
	}
	Settings.PatternLists["1"].Name = Strings["default_emote_set"];
end

if Settings.NameLists == nil then
	Settings.NextNameListId = "1";
	Settings.NameLists = {};
end

for channelKey, channelType in pairs(ChannelTypes) do
	if Settings.ChannelData[channelType] == nil then
		Settings.ChannelData[channelType] =
		{
			Enabled = false,
			Colour = BabbleOptions.DefaultColour,
			CurrentPatternList = nil,
			CurrentNameList = nil,
			RouteToStandard = true
		}
		if channelKey == Turbine.ChatType.Emote then
			Settings.ChannelData[channelType].Enabled = true;
			Settings.ChannelData[channelType].CurrentPatternList = "1";
			Settings.ChannelData[channelType].CurrentNameList = "0";
		elseif channelKey == Turbine.ChatType.Say then
			Settings.ChannelData[channelType].BlockNPC = false;
		end
	end
	
	if Settings.ChannelData[channelType].RouteToStandard == nil then
		Settings.ChannelData[channelType].RouteToStandard = true;
	end
end

for channelKey, channelType in pairs(OtherChannelTypes) do
	if Settings.ChannelData[channelType] == nil then
		Settings.ChannelData[channelType] =
		{
			Colour = BabbleOptions.DefaultColour,
		}
	end
end

if Settings.EnableChatWindow == nil then
	Settings.EnableChatWindow = false;
end

if Settings.ShowChatWindow == nil or Settings.AutoHiddenChatWindow then
	Settings.ShowChatWindow = true;
	Settings.AutoHiddenChatWindow = nil;
end

if Settings.EnableBabbleButton == nil then
	Settings.EnableBabbleButton = true;
end

if Settings.SettingsWindow == nil then
	Settings.SettingsWindow =
	{
		Position = {},
		Size = {}
	}
end

if Settings.ColoursWindow == nil then
	Settings.ColoursWindow =
	{
		Position = {}
	}
end

if Settings.PatternListWindow == nil then
	Settings.PatternListWindow =
	{
		Position = {},
		Size = {}
	}
end

if Settings.NameListWindow == nil then
	Settings.NameListWindow = 
	{
		Position = {},
		Size = {}
	}
end

if Settings.StatusWindow == nil then
	Settings.StatusWindow = 
	{
		Position = {}
	}
end

if Settings.ChatWindow == nil then
	Settings.ChatWindow = 
	{
		Position = {},
		Size = {}
	}
end

if Settings.StatusButton == nil then
	Settings.StatusButton = 
	{
		Position = {}
	}
end

if Settings.LinesToSave == nil then
	Settings.LinesToSave = "100";
end

if Settings.ChatWindowOpacity == nil then
	Settings.ChatWindowOpacity = "96"
end

if Settings.ShowTimestamp == nil then
	Settings.ShowTimestamp = false
end

if Settings.ChatTabs == nil then
	Settings.ChatTabs = {};
	CreateBabbleTabSettings("1", Strings["tab_name_main"], false);
	Settings.ChatTabs["1"].Filtered[ChannelTypes[Turbine.ChatType.Emote]] = true;
end

Settings.CurrentSet = nil;
Settings.FilterSets = nil;

BabbleNPC = false;

BabbleHelpers = {};

for _, channelType in pairs(ChannelTypes) do
	local channelData = Settings.ChannelData[channelType];
	BabbleHelpers[channelType] = BabbleHelper(channelType, channelData.Enabled, channelData.CurrentPatternList, channelData.CurrentNameList);
end

StatusWindow = StatusWindow();
StatusWindow:EnableBabbleCheckBox(BabbleOptions.Enabled);

ChatWindow = ChatWindow();
if Settings.EnableChatWindow and Settings.ShowChatWindow then
	ChatWindow:Show();
end

StatusButton = Simbo.Babble.StatusButton();
StatusButton:SetVisible(Settings.EnableBabbleButton);
StatusButton:SetZOrder(StatusWindow:GetZOrder() + 10);

local optionsPanel = OptionsPanel();
plugin.GetOptionsPanel = function()
    return optionsPanel;
end

BabbleCommand = Turbine.ShellCommand();

function BabbleCommand:Execute(cmd, args)
	if args ~= nil then
		args = args:lower();
	end

	if args == Strings["cmd_babble_on"] then
		EnableBabble(true, true);
	elseif args == Strings["cmd_babble_off"] then
		EnableBabble(false, true);
	elseif args == Strings["cmd_babble_edit"] or args == Strings["cmd_babble_status"] then
		ShowBabbleStatus();
		StatusWindow:Show();
	elseif args == Strings["cmd_babble_list"] then
		ListBabbleFilters();
	else
		BabbleCommand:GetHelp();
	end
end

function BabbleCommand:GetHelp()
	ShowBabbleMessage(Strings["babble_help_on"]);
	ShowBabbleMessage(Strings["babble_help_off"]);
	ShowBabbleMessage(Strings["babble_help_list"]);
	ShowBabbleMessage(Strings["babble_help_status"]);
	ShowBabbleMessage("");
	ShowBabbleStatus();
end

function Babble_ChatReceived(sender, args)
	if BabbleOptions.Enabled then
		local message = args.Message;
		if message:match(".+\n$") then
			message = message:sub(1, message:len() - 1);
		end
	
		local channelType = ChannelTypes[args.ChatType];
		if channelType ~= nil then
			BabbleHelpers[channelType]:ProcessMessage(message);
		else
			local otherChannelType = OtherChannelTypes[args.ChatType];
			if otherChannelType ~= nil then
				ChatWindow:AddLine(message, otherChannelType, "Other");
			end
		end
	end
end

AddCallback(Turbine.Chat, "Received", Babble_ChatReceived);

Plugins["Babble"].Unload = function(sender, args)
	RemoveCallback(Turbine.Chat, "Received", Babble_ChatReceived);
	
	StatusWindow:SavePositionSettings();
	StatusButton:SavePositionSettings();
	ChatWindow:SavePositionSettings();
	
	SaveBabbleSettings(true);
end

Turbine.Shell.AddCommand(Strings["babble"], BabbleCommand);
ShowBabbleMessage(Strings["babble_version"]..Plugins["Babble"]:GetVersion().." - Simbo");
ShowBabbleMessage(Strings["babble_welcome"]);
ShowBabbleStatus();
