import "Simbo.Babble.WindowLotro";
import "Simbo.Babble.StyledList";
import "Simbo.Babble.ConfirmationWindow";

NameListWindow = class(WindowLotro);

local Constants =
{
	MinimumSize =
	{
		Width = 320,
		Height = 400
	},
	Padding =
	{
		Top = 50,
		Left = 20,
		Bottom = 20,
		Right = 20
	},
	LineSpacing = 8,
	ColumnSpacing = 8,
	Sizes =
	{
		LabelWidth = UIOptions["wide_name_list_label"] and 95 or 75, 
		ListBoxWidth = 168
	}
}

function NameListWindow:Constructor()
	WindowLotro.Constructor(self);
	
	self.ListId = nil;
	self.ListName = nil;
	self.Changed = false;
	self.CurrentType = nil;
	
	self:SetText(Strings["named_players"]);
	self:SetPosition(tonumber(Settings.NameListWindow.Position.Left) or 200, tonumber(Settings.NameListWindow.Position.Top) or 200);
	self:SetSize(tonumber(Settings.NameListWindow.Size.Width) or Constants.MinimumSize.Width, tonumber(Settings.NameListWindow.Size.Height) or Constants.MinimumSize.Height);
	self:SetOpacity(Globals.Opacity);

	if self:GetWidth() < Constants.MinimumSize.Width then
		self:SetWidth(Constants.MinimumSize.Width);
	end
	if self:GetHeight() < Constants.MinimumSize.Height then
		self:SetHeight(Constants.MinimumSize.Height);
	end
	
	self.AddWindow = ConfirmationWindow();
	self.AddWindow:SetZOrder(self:GetZOrder() + 10);
	self.AddWindow.Tag = nil;
	self.AddWindow.ButtonClick = function (sender,args)
		local text = args.Text:Trim();
		return self:SaveName(text, self.AddWindow.Tag);
	end

	local top = Constants.Padding.Top;
	local left = Constants.Padding.Left;
	
	self.NameListLabel = CreateLabel(self, true);
	self.NameListLabel:SetSize(Constants.Sizes.ListBoxWidth, Globals.Sizes.Label.Height);
	self.NameListLabel:SetText(Strings["player_names"]);
	self.NameListLabel:SetPosition(left, top);

	top = top + self.NameListLabel:GetHeight();
	
	self.NameList = StyledList(self);
	self.NameList:SetPosition(left, top);
	self.NameList:SetWidth(Constants.Sizes.ListBoxWidth);
	self.NameList:SetBorderColor(Globals.BorderColours.ListBox);
	self.NameList:SetBackColor(Globals.BackgroundColours.ListBox);
	self.NameList:SetForeColor(Globals.Colours.ListBox);
	self.NameList:SetFont(Globals.Fonts.ListBox);
	self.NameList:SetOpacity(Globals.Opacity);
	self.NameList.SelectedIndexChanged = function(sender, args)
		self.EditNameButton:SetEnabled(args.SelectedIndex > 0);
		self.RemoveNameButton:SetEnabled(args.SelectedIndex > 0);
		if self.AddWindow.Tag == "EDITNAME" then
			self.AddWindow:SetVisible(false);
			self.AddWindow.Tag = nil;
		end
	end
	
	left = left + self.NameList:GetWidth() + Constants.ColumnSpacing;
	
	self.AddNameButton = CreateButton(self);
	self.AddNameButton:SetPosition(left, top);
	self.AddNameButton:SetText(Strings["add"]);
	self.AddNameButton.MouseClick = function(sender, args)
		self.AddWindow.Tag = "ADDNAME";
		self.AddWindow:ShowWindow(Strings["add_name_title"], Strings["add_name_message"] .. "\n" .. Strings["add_name_message_comma"], true, false);
	end
	
	top = top + self.AddNameButton:GetHeight() + Constants.LineSpacing;
	
	self.EditNameButton = CreateButton(self);
	self.EditNameButton:SetPosition(left, top);
	self.EditNameButton:SetText(Strings["edit"]);
	self.EditNameButton.MouseClick = function(sender, args)
		local item = self.NameList:GetSelectedItem();
		if item ~= nil then
			self.AddWindow.Tag = "EDITNAME";
			self.AddWindow:ShowWindow(Strings["edit_name_title"], Strings["add_name_message"], true, false, item);
		end
	end	
	
	top = top + self.EditNameButton:GetHeight() + Constants.LineSpacing;
	
	self.RemoveNameButton = CreateButton(self);
	self.RemoveNameButton:SetPosition(left, top);
	self.RemoveNameButton:SetText(Strings["remove"]);
	self.RemoveNameButton.MouseClick = function(sender, args)
		self.Changed = true;
		self.NameList:RemoveSelectedItem();
		self.RemoveNameButton:SetEnabled(false);
	end

	left = Constants.Padding.Left;
	
	self.ListNameLabel = CreateLabel(self);
	self.ListNameLabel:SetText(Strings["list_name"]);
	self.ListNameLabel:SetLeft(left);
	self.ListNameLabel:SetWidth(Constants.Sizes.LabelWidth);
	
	self.ListNameBox = CreateTextBox(self);
	self.ListNameBox:SetLeft(self.ListNameLabel:GetLeft() + self.ListNameLabel:GetWidth() + Constants.ColumnSpacing);
	
	self.WarningLabel = CreateLabel(self, true);
	self.WarningLabel:SetLeft(left);
	self.WarningLabel:SetHeight(self.WarningLabel:GetHeight() * 2);
	self.WarningLabel:SetForeColor(Globals.Colours.Warning);
	self.WarningLabel:SetText(Strings["list_in_use"]);
	self.WarningLabel:SetVisible(false);
	
	self.CancelButton = CreateButton(self);
	self.CancelButton:SetText(Strings["cancel"]);
	self.CancelButton.MouseClick = function(sender, args)
		self:Hide();
	end
	
	self.SaveButton = CreateButton(self);
	self.SaveButton:SetText(Strings["ok"]);
	self.SaveButton.MouseClick = function(sender, args)
		if args.Button == Turbine.UI.MouseButton.Left then
			local names = self:SaveNameSettings();
			if names ~= nil then
				args = { Id = self.ListId, List = names, Changed = (self.Changed or (self.ListName ~= names.Name)) };
				self.Saved(self, args);
				self:Hide();
			end
		end
	end
	
	self.VisibleChanged = function(sender, args)
		if not self:IsVisible() and not self.UIHidden then
			self.AddWindow:SetVisible(false);
		else
			self:CheckWindowPosition();
		end
	end
	
	self.Saved = function() end;
		
	self:LayoutControls();
	self:SetResizer(Constants.MinimumSize.Width, Constants.MinimumSize.Height);
end

function NameListWindow:LayoutControls()
	local listBoxHeight = self:GetHeight() - self.NameList:GetTop() - Constants.Padding.Bottom - self.ListNameLabel:GetHeight() - self.WarningLabel:GetHeight() - self.CancelButton:GetHeight() - Constants.LineSpacing * 3;
	local listBoxWidth = self:GetWidth() - self.NameList:GetLeft() - self.AddNameButton:GetWidth() - Constants.ColumnSpacing - Constants.Padding.Right;
	self.NameList:SetHeight(listBoxHeight);
	self.NameList:SetWidth(listBoxWidth);

	self.AddNameButton:SetLeft(self.NameList:GetLeft() + self.NameList:GetWidth() + Constants.ColumnSpacing);
	self.EditNameButton:SetLeft(self.NameList:GetLeft() + self.NameList:GetWidth() + Constants.ColumnSpacing);
	self.RemoveNameButton:SetLeft(self.NameList:GetLeft() + self.NameList:GetWidth() + Constants.ColumnSpacing);
	
	self.ListNameLabel:SetTop(self.NameList:GetTop() + self.NameList:GetHeight() + Constants.LineSpacing);
	self.ListNameBox:SetTop(self.ListNameLabel:GetTop());
	self.ListNameBox:SetWidth(self:GetWidth() - self.ListNameBox:GetLeft() - Constants.Padding.Right);
	
	self.WarningLabel:SetTop(self.ListNameBox:GetTop() + self.ListNameBox:GetHeight() + Constants.LineSpacing);
	self.WarningLabel:SetWidth(self:GetWidth() - Constants.Padding.Left - Constants.Padding.Right);
	
	self.CancelButton:SetPosition((self:GetWidth() - (self.CancelButton:GetWidth() + Constants.ColumnSpacing + self.CancelButton:GetWidth())) / 2, self:GetHeight() - Constants.Padding.Bottom - self.CancelButton:GetHeight());
	self.SaveButton:SetPosition(self.CancelButton:GetLeft() + self.CancelButton:GetWidth() + Constants.ColumnSpacing, self.CancelButton:GetTop());
end

function NameListWindow:Show(listId, names, listName, currentType)
	self.ListId = listId;
	self.ListName = listName;
	self.CurrentType = currentType;
	
	self:PopulateSettings(names);
	self.ListNameBox:SetText(listName);
	
	local listInUse = false;
	if listId ~= nil then
		for key, data in pairs(Settings.ChannelData) do
			if key ~= self.CurrentType and listId == data.CurrentNameList then
				listInUse = true;
			end
		end
	end
	self.WarningLabel:SetVisible(listInUse);
	
	self.AddWindow:SetVisible(false);
	self:SetVisible(true);
	self:Activate();
end

function NameListWindow:Hide()
	self.AddWindow:Close();
	self:Close();
end

function NameListWindow:PopulateSettings(names)
	self.NameList:ClearItems();
	
	self.EditNameButton:SetEnabled(false);
	self.RemoveNameButton:SetEnabled(false);
	
	local itemIndex = 1;
	local done = false;
	while not done do
		local stringIndex = tostring(itemIndex);
		local from = names[stringIndex];
		if from ~= nil then
			self.NameList:AddItem(from);
			itemIndex = itemIndex + 1;
		else
			done = true;
		end
	end
end

function NameListWindow:SaveName(text, mode)
	if string.len(text) > 0 then
		self.Changed = true;
		if mode == "ADDNAME" then
			local names = text:Split(",");
			local namesAdded = false;
			local nameCount = 0;
			for _, name in ipairs(names) do
				nameCount = nameCount + 1;
				local checkIndex = self.NameList:FindInsertIndex(name);
				if checkIndex ~= nil then
					self.NameList:AddItem(name);
					namesAdded = true;
				end
			end
			if not namesAdded then
				if nameCount == 1 then
					ShowBabbleMessage(Strings["name_exists"], 2);
				else
					ShowBabbleMessage(Strings["names_exists"], 2);
				end
				return false;
			end
		elseif mode == "EDITNAME" then
			if text:find(",") then
				ShowBabbleMessage(Strings["name_no_commas"], 2);
				return false;
			end
			local item = self.NameList:GetSelectedItem();
			if item ~= nil and item:Compare(text) ~= 0 then
				local checkIndex = self.NameList:FindInsertIndex(text);
				if checkIndex == nil then
					ShowBabbleMessage(Strings["name_exists"], 2);
				return false;
				end
				self.NameList:RemoveSelectedItem();
				self.NameList:AddItem(text, true);
			end
		end
	end
	
	return true;
end

function NameListWindow:SavePositionSettings()
	Settings.NameListWindow.Position.Top = tostring(self:GetTop());
	Settings.NameListWindow.Position.Left = tostring(self:GetLeft());
	Settings.NameListWindow.Size.Width = tostring(self:GetWidth());
	Settings.NameListWindow.Size.Height = tostring(self:GetHeight());
end

function NameListWindow:SaveNameSettings()
	local setName = self.ListNameBox:GetText():Trim();
	if setName == "" then
		ShowBabbleMessage(Strings["list_name_required"], 2);
		self.ListNameBox:Focus();
		return nil;
	end
	
	local setDetails = { ["Name"] = setName };
	
	local names = self.NameList:GetAllItems();
	itemIndex = 1;
	done = false;
	while not done do
		local stringIndex = tostring(itemIndex);
		local name = names[stringIndex];
		if name ~= nil then
			setDetails[stringIndex] = name;
			itemIndex = itemIndex + 1;
		else
			done = true;
		end
	end
	
	if itemIndex == 1 then
		ShowBabbleMessage(Strings["at_least_one_name"], 2);
		return nil;
	end
	
	return setDetails;
end