import "Simbo.Babble.WindowLotro";
import "Simbo.Babble.StyledList";
import "Simbo.Babble.PatternList";
import "Simbo.Babble.ConfirmationWindow";

PatternListWindow = class(WindowLotro);

local Constants =
{
	MinimumSize =
	{
		Width = 500,
		Height = 500
	},
	Padding =
	{
		Top = 50,
		Left = 20,
		Bottom = 20,
		Right = 20
	},
	LineSpacing = 8,
	ColumnSpacing = 8,
	Sizes =
	{
		LabelWidth = UIOptions["wide_name_list_label"] and 95 or 75,
		CheckBoxWidth = 80,
		Dropdown = 
		{
			Width = 150,
			Height = 20,
			Open = 120
		},
	}
}

function PatternListWindow:Constructor()
	WindowLotro.Constructor(self);
	
	self.ListId = nil;
	self.ListName = nil;
	self.Changed = false;
	self.CurrentType = nil;
	
	self:SetText(Strings["pattern_list"]);
	self:SetPosition(tonumber(Settings.PatternListWindow.Position.Left) or 150, tonumber(Settings.PatternListWindow.Position.Top) or 150);
	self:SetSize(tonumber(Settings.PatternListWindow.Size.Width) or Constants.MinimumSize.Width, tonumber(Settings.PatternListWindow.Size.Height) or Constants.MinimumSize.Height);
	self:SetOpacity(Globals.Opacity);
		
	self.AddWindow = ConfirmationWindow();
	self.AddWindow:SetZOrder(self:GetZOrder() + 10);
	self.AddWindow.Tag = nil;
	self.AddWindow.ButtonClick = function (sender,args)
		local text = args.Text:Trim();
		return self:SavePattern(text, self.AddWindow.Tag);
	end

	local top = Constants.Padding.Top;
	local left = Constants.Padding.Left;
	
	local filterTypeLabel = CreateLabel(self, true);
	filterTypeLabel:SetWidth(Constants.Sizes.LabelWidth);
	filterTypeLabel:SetTextAlignment(Turbine.UI.ContentAlignment.LeftCenter);
	filterTypeLabel:SetText("Filter Type");
	filterTypeLabel:SetPosition(left, top);
	
	left = left + filterTypeLabel:GetWidth() + Constants.ColumnSpacing;
	
	self.PatternTypeDropdown = CreateDropdown(self, left, top, Constants.Sizes.Dropdown.Width, Constants.Sizes.Dropdown.Height, Constants.Sizes.Dropdown.Open);
	self.PatternTypeDropdown:AddItem(Strings["block_patterns"], "B");
	self.PatternTypeDropdown:AddItem(Strings["allow_patterns"], "A");
	self.PatternTypeDropdown.SelectedIndexChanged = function(sender, args)
		self.Changed = true;
	end
	
	top = top + filterTypeLabel:GetHeight();
	left = Constants.Padding.Left;
	
	self.UnlessFromLabel = CreateLabel(self, true);
	self.UnlessFromLabel:SetSize(Constants.Sizes.CheckBoxWidth, Globals.Sizes.Label.Height * 1.5);
	self.UnlessFromLabel:SetFont(Globals.Fonts.SmallLabel);
	self.UnlessFromLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter);
	self.UnlessFromLabel:SetText(Strings["unless_from"]);
	self.UnlessFromLabel:SetTop(top);
	
	self.UnlessToLabel = CreateLabel(self, true);
	self.UnlessToLabel:SetSize(Constants.Sizes.CheckBoxWidth, Globals.Sizes.Label.Height * 1.5);
	self.UnlessToLabel:SetFont(Globals.Fonts.SmallLabel);
	self.UnlessToLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter);
	self.UnlessToLabel:SetText(Strings["unless_to"]);
	self.UnlessToLabel:SetTop(top);
	
	top = top + self.UnlessFromLabel:GetHeight();
	
	self.PatternLabel = CreateLabel(self);
	self.PatternLabel:SetText(Strings["pattern"]);
	self.PatternLabel:SetPosition(left, top);
	
	self.UnlessFromCheckBox = CreateCheckBox(self)
	self.UnlessFromCheckBox:SetTop(top);
	self.UnlessFromCheckBox.IgnoreEvent = false;
	self.UnlessFromCheckBox.CheckedChanged = function(sender, args)
		self.Changed = true;
		if not self.UnlessFromCheckBox.IgnoreEvent then
			self.PatternList:SetAllCheckboxes(1, self.UnlessFromCheckBox:IsChecked());
		else
			self.UnlessFromCheckBox.IgnoreEvent = false;
		end
	end
	
	self.UnlessToCheckBox = CreateCheckBox(self)
	self.UnlessToCheckBox:SetTop(top);
	self.UnlessToCheckBox.IgnoreEvent = false;
	self.UnlessToCheckBox.CheckedChanged = function(sender, args)
		self.Changed = true;
		if not self.UnlessToCheckBox.IgnoreEvent then
			self.PatternList:SetAllCheckboxes(2, self.UnlessToCheckBox:IsChecked());
		else
			self.UnlessToCheckBox.IgnoreEvent = false;
		end
	end
	
	top = top + self.PatternLabel:GetHeight();
	
	self.PatternList = PatternList(self, 2);
	self.PatternList:SetPosition(left, top);
	self.PatternList:SetBorderColor(Globals.BorderColours.ListBox);
	self.PatternList:SetBackColor(Globals.BackgroundColours.ListBox);
	self.PatternList:SetForeColor(Globals.Colours.ListBox);
	self.PatternList:SetFont(Globals.Fonts.ListBox);
	self.PatternList:SetOpacity(Globals.Opacity);
	self.PatternList.SelectedIndexChanged = function(sender, args)
		self.EditPatternButton:SetEnabled(args.SelectedIndex > 0);
		self.RemovePatternButton:SetEnabled(args.SelectedIndex > 0);
		if self.AddWindow.Tag == "EDITPATTERN" then
			self.AddWindow:SetVisible(false);
			self.AddWindow.Tag = nil;
		end
	end
	self.PatternList.CheckBoxSelected = function(sender, args)
		self.Changed = true;
		
		local box = nil;
		if args.Index == 1 then
			box = self.UnlessFromCheckBox;
		else
			box = self.UnlessToCheckBox;
		end
		if args.AllSelected ~= box:IsChecked() then
			box.IgnoreEvent = true;
			box:SetChecked(args.AllSelected);
		end
	end
	
	self.AddPatternButton = CreateButton(self, 0);
	self.AddPatternButton:SetText(Strings["add"]);
	self.AddPatternButton:SetLeft(left);
	self.AddPatternButton.MouseClick = function(sender, args)
		self.AddWindow.Tag = "ADDPATTERN";
		self.AddWindow:ShowWindow(Strings["add_pattern_title"], Strings["add_pattern_message"], true, false);
	end
	
	self.EditPatternButton = CreateButton(self, 0);
	self.EditPatternButton:SetText(Strings["edit"]);
	self.EditPatternButton:SetLeft(left + self.AddPatternButton:GetWidth() + Constants.ColumnSpacing);
	self.EditPatternButton.MouseClick = function(sender, args)
		local item = self.PatternList:GetSelectedItem();
		if item ~= nil then
			self.AddWindow.Tag = "EDITPATTERN";
			self.AddWindow:ShowWindow(Strings["edit_pattern_title"], Strings["add_pattern_message"], true, false, item.Text);
		end
	end
	
	self.RemovePatternButton = CreateButton(self, 0);
	self.RemovePatternButton:SetText(Strings["remove"]);
	self.RemovePatternButton:SetLeft(self.EditPatternButton:GetLeft() + self.EditPatternButton:GetWidth() + Constants.ColumnSpacing);
	self.RemovePatternButton.MouseClick = function(sender, args)
		self.Changed = true;
		self.PatternList:RemoveSelectedItem();
		self.EditPatternButton:SetEnabled(false);
		self.RemovePatternButton:SetEnabled(false);
		if self.AddWindow.Tag == "EDITPATTERN" then
			self.AddWindow:SetVisible(false);
			self.AddWindow.Tag = nil;
		end
	end

	self.ClearListButton = CreateButton(self, UIOptions["wide_clear_list_button"] and 1 or 0);
	self.ClearListButton:SetText(Strings["clear_list"]);
	self.ClearListButton.MouseClick = function(sender, args)
		self.Changed = true;
		self.PatternList:ClearItems();
		self.EditPatternButton:SetEnabled(false);
		self.RemovePatternButton:SetEnabled(false);
		if self.AddWindow.Tag == "EDITPATTERN" then
			self.AddWindow:SetVisible(false);
			self.AddWindow.Tag = nil;
		end
	end
	
	self.PatternListNameLabel = CreateLabel(self);
	self.PatternListNameLabel:SetText(Strings["list_name"]);
	self.PatternListNameLabel:SetLeft(left);
	self.PatternListNameLabel:SetWidth(Constants.Sizes.LabelWidth);
	
	self.PatternListNameBox = CreateTextBox(self);
	self.PatternListNameBox:SetLeft(self.PatternListNameLabel:GetLeft() + self.PatternListNameLabel:GetWidth() + Constants.ColumnSpacing);
	
	self.WarningLabel = CreateLabel(self, true);
	self.WarningLabel:SetLeft(left);
	self.WarningLabel:SetHeight(self.WarningLabel:GetHeight() * 2);
	self.WarningLabel:SetForeColor(Globals.Colours.Warning);
	self.WarningLabel:SetText(Strings["list_in_use"]);
	self.WarningLabel:SetVisible(false);

	self.CancelButton = CreateButton(self);
	self.CancelButton:SetText(Strings["cancel"]);
	self.CancelButton.MouseClick = function(sender, args)
		self:Hide();
	end
	
	self.SaveButton = CreateButton(self);
	self.SaveButton:SetText(Strings["ok"]);
	self.SaveButton.MouseClick = function(sender, args)
		if args.Button == Turbine.UI.MouseButton.Left then
			local patternList = self:SavePatternSettings();
			if patternList ~= nil then
				args = { Id = self.ListId, List = patternList, Changed = (self.Changed or (self.ListName ~= patternList.Name)) };
				self.Saved(self, args);
				self:Hide();
			end
		end
	end
	
	self.VisibleChanged = function(sender, args)
		if not self:IsVisible() and not self.UIHidden then
			self.AddWindow:SetVisible(false);
		else
			self:CheckWindowPosition();
		end
	end
	
	self.Saved = function() end;
		
	self:LayoutControls();
	self:SetResizer(Constants.MinimumSize.Width, Constants.MinimumSize.Height);
end

function PatternListWindow:LayoutControls()
	local listBoxWidth = self:GetWidth() - Constants.Padding.Left - Constants.Padding.Right;
	local listBoxHeight = self:GetHeight() - self.PatternList:GetTop() - Constants.Padding.Bottom - self.AddPatternButton:GetHeight() - self.PatternListNameLabel:GetHeight() - self.WarningLabel:GetHeight() - self.CancelButton:GetHeight() - Constants.LineSpacing * 4;

	self.PatternList:SetSize(listBoxWidth, listBoxHeight, Constants.Sizes.CheckBoxWidth);
	
	self.PatternLabel:SetWidth(self.PatternList:GetLabelWidth());
	self.UnlessFromLabel:SetLeft(self.PatternLabel:GetLeft() + self.PatternLabel:GetWidth() - (Constants.Sizes.CheckBoxWidth - Globals.Sizes.CheckBoxOnly.Width) / 2);
	self.UnlessFromLabel:SetWidth(Constants.Sizes.CheckBoxWidth);
	self.UnlessFromCheckBox:SetLeft(self.PatternLabel:GetLeft() + self.PatternLabel:GetWidth());
	self.UnlessToLabel:SetLeft(self.UnlessFromLabel:GetLeft() + self.UnlessFromLabel:GetWidth());
	self.UnlessToLabel:SetWidth(Constants.Sizes.CheckBoxWidth);
	self.UnlessToCheckBox:SetLeft(self.UnlessFromCheckBox:GetLeft() + self.UnlessFromLabel:GetWidth());
	
	self.AddPatternButton:SetTop(self.PatternList:GetTop() + self.PatternList:GetHeight() + Constants.LineSpacing);
	self.EditPatternButton:SetTop(self.AddPatternButton:GetTop());
	self.RemovePatternButton:SetTop(self.AddPatternButton:GetTop());
	self.ClearListButton:SetPosition(self:GetWidth() - Constants.Padding.Right - self.ClearListButton:GetWidth(), self.AddPatternButton:GetTop())

	self.PatternListNameLabel:SetTop(self.AddPatternButton:GetTop() + self.AddPatternButton:GetHeight() + Constants.LineSpacing);
	self.PatternListNameBox:SetTop(self.PatternListNameLabel:GetTop());
	
	self.WarningLabel:SetTop(self.PatternListNameBox:GetTop() + self.PatternListNameBox:GetHeight() + Constants.LineSpacing);
	self.WarningLabel:SetWidth(listBoxWidth);
	
	self.CancelButton:SetPosition((self:GetWidth() - (self.CancelButton:GetWidth() + Constants.ColumnSpacing + self.CancelButton:GetWidth())) / 2, self:GetHeight() - Constants.Padding.Bottom - self.CancelButton:GetHeight());
	self.SaveButton:SetPosition(self.CancelButton:GetLeft() + self.CancelButton:GetWidth() + Constants.ColumnSpacing, self.CancelButton:GetTop());
	
	self.PatternListNameBox:SetWidth(self.SaveButton:GetLeft() + self.SaveButton:GetWidth() - self.PatternListNameBox:GetLeft());
end

function PatternListWindow:Show(listId, blockedPatterns, listName, currentType)
	self.ListId = listId;
	self.ListName = listName;
	self.Changed = false;
	self.CurrentType = currentType;

	self:PopulateSettings(blockedPatterns);
	self.PatternListNameBox:SetText(listName);
	self.UnlessFromCheckBox:SetChecked(self.PatternList:GetItemCount() > 0 and self.PatternList:IsAllSelected(1));
	self.UnlessToCheckBox:SetChecked(self.PatternList:GetItemCount() > 0 and self.PatternList:IsAllSelected(2));
	
	local listInUse = false;
	if listId ~= nil then
		for key, data in pairs(Settings.ChannelData) do
			if key ~= self.CurrentType and listId == data.CurrentPatternList then
				listInUse = true;
			end
		end
	end
	self.WarningLabel:SetVisible(listInUse);

	self.AddWindow:SetVisible(false);
	self:SetVisible(true);
	self:Activate();
end

function PatternListWindow:Hide()
	self.AddWindow:Close();
	self:Close();
end

function PatternListWindow:PopulateSettings(blockedPatterns)
	self.PatternList:ClearItems();
	
	self.PatternTypeDropdown:SetSelectedValue(blockedPatterns.Type or "B");
	self.EditPatternButton:SetEnabled(false);
	self.RemovePatternButton:SetEnabled(false);
	
	local itemIndex = 1;
	local done = false;
	while not done do
		local stringIndex = tostring(itemIndex);
		local filterItem = blockedPatterns[stringIndex];
		if filterItem ~= nil then
			self.PatternList:AddItem(filterItem.Text, { ["1"] = filterItem.UnlessFrom, ["2"] = filterItem.UnlessTo });
			itemIndex = itemIndex + 1;
		else
			done = true;
		end
	end
end

function PatternListWindow:SavePattern(text, mode)
	if string.len(text) > 0 then
		self.Changed = true;
		if mode == "ADDPATTERN" then
			local checkIndex = self.PatternList:FindInsertIndex(text);
			if checkIndex == nil then
				ShowBabbleMessage(Strings["pattern_exists"], 2);
				return false;
			end
			self.PatternList:AddItem(text, { ["1"] = self.UnlessFromCheckBox:IsChecked(), ["2"] = self.UnlessToCheckBox:IsChecked() }, false);
		elseif mode == "EDITPATTERN" then
			local item = self.PatternList:GetSelectedItem();
			if item ~= nil and item.Text:Compare(text) ~= 0 then
				local checkIndex = self.PatternList:FindInsertIndex(text);
				if checkIndex == nil then
					ShowBabbleMessage(Strings["pattern_exists"], 2);
					return false;
				end
				self.PatternList:RemoveSelectedItem();
				self.PatternList:AddItem(text, { ["1"] = item.CheckBoxes["1"], ["2"] = item.CheckBoxes["2"] }, true);
			end
		end
	end
	
	return true;
end

function PatternListWindow:SavePositionSettings()
	Settings.PatternListWindow.Position.Top = tostring(self:GetTop());
	Settings.PatternListWindow.Position.Left = tostring(self:GetLeft());
	Settings.PatternListWindow.Size.Width = tostring(self:GetWidth());
	Settings.PatternListWindow.Size.Height = tostring(self:GetHeight());
end

function PatternListWindow:SavePatternSettings()
	local setName = self.PatternListNameBox:GetText():Trim();
	if setName == "" then
		ShowBabbleMessage(Strings["list_name_required"], 2);
		self.PatternListNameBox:Focus();
		return nil;
	end
	
	local setDetails = { ["Name"] = setName, ["Type"] = self.PatternTypeDropdown:GetSelectedValue() };
	
	local patterns = self.PatternList:GetAllItems();
	local itemIndex = 1;
	local done = false;
	while not done do
		local stringIndex = tostring(itemIndex);
		local pattern = patterns[stringIndex];
		if pattern ~= nil then
			setDetails[stringIndex] =
			{
				Text = pattern.Text,
				UnlessFrom = pattern.CheckBoxes["1"],
				UnlessTo = pattern.CheckBoxes["2"]
			}
			itemIndex = itemIndex + 1;
		else
			done = true;
		end
	end
	
	if itemIndex == 1 then
		ShowBabbleMessage(Strings["at_least_one_pattern"], 2);
		return nil;
	end
	
	return setDetails;
end