import "Simbo.Babble.WindowLotro";
import "Simbo.Babble.DropdownList";
import "Simbo.Babble.PatternListWindow";
import "Simbo.Babble.NameListWindow";
import "Simbo.Babble.ConfirmationWindow";

SettingsWindow = class(WindowLotro);

local Constants =
{
	MinimumSize =
	{
		Width = UIOptions["wide_settings_label"] and 445 or 440,
		Height = 370
	},
	Padding =
	{
		Top = 50,
		Left = 20,
		Bottom = 20,
		Right = 20
	},
	Sizes =
	{
		LabelWidth = UIOptions["wide_settings_label"] and 115 or 110,
		Dropdown = 
		{
			Width = 200,
			Height = 20,
			Open = 120
		},
		CheckBox = 350
	},
	LineSpacing = 6,
	ColumnSpacing = 6
}

function SettingsWindow:Constructor()
	WindowLotro.Constructor(self);

	self.ChannelType = nil;
	self.ChangedPatternLists = {};
	self.ChangedNameLists = {};
	self.NextPatternListId = tonumber(Settings.NextPatternListId);
	self.NextNameListId = tonumber(Settings.NextNameListId);
	self.PatternsSaved = false;
	self.NamesSaved = false;
	self.StandardCheckBoxChanged = false;
	self.ExtraCheckBoxChanged = false;
	self.CopyMode = false;
	self.CancelPressed = false;
	self.SavePressed = false;
	
	self:SetText(Strings["settings"]);
	self:SetPosition(tonumber(Settings.SettingsWindow.Position.Left) or 100, tonumber(Settings.SettingsWindow.Position.Top) or 100);
	self:SetSize(tonumber(Settings.SettingsWindow.Size.Width) or Constants.MinimumSize.Width, tonumber(Settings.SettingsWindow.Size.Height) or Constants.MinimumSize.Height);
	self:SetOpacity(Globals.Opacity);
	if self:GetWidth() < Constants.MinimumSize.Width then
		self:SetWidth(Constants.MinimumSize.Width);
	end
	if self:GetHeight() < Constants.MinimumSize.Height then
		self:SetHeight(Constants.MinimumSize.Height);
	end
	
	self.PatternListWindow = PatternListWindow();
	self.PatternListWindow.Saved = function (sender, args)
		self.PatternsSaved = self.PatternsSaved or args.Changed;
		self.ChangedPatternLists[args.Id] = args.List;
		if args.Id == tostring(self.NextPatternListId) then
			self.NextPatternListId = self.NextPatternListId + 1;
			self.PatternListDropdown:AddItem(args.List.Name, args.Id, true);
			self:SetPatternButtonStatus();
		elseif self.PatternListDropdown:GetSelectedText() ~= args.List.Name then
			self.PatternListDropdown:UpdateSelectedItem(args.List.Name, args.Id);
		end
		if args.Changed then
			self:SetSaveButtonsStatus(true);
		end
	end
	
	self.NameListWindow = NameListWindow();
	self.NameListWindow.Saved = function (sender, args)
		self.NamesSaved = self.NamesSaved or args.Changed;
		self.ChangedNameLists[args.Id] = args.List;
		if args.Id == tostring(self.NextNameListId) then
			self.NextNameListId = self.NextNameListId + 1;
			self.NameListDropdown:AddItem(args.List.Name, args.Id, true);
		elseif self.NameListDropdown:GetSelectedText() ~= args.List.Name then
			self.NameListDropdown:UpdateSelectedItem(args.List.Name, args.Id);
		end
		if args.Changed then
			self:SetSaveButtonsStatus(true);
		end
	end

	local top = Constants.Padding.Top;
	local left = Constants.Padding.Left;
	
	local channelHeaderLabel = CreateLabel(self, false);
	channelHeaderLabel:SetPosition(left, top);
	channelHeaderLabel:SetWidth(Constants.Sizes.LabelWidth);
	channelHeaderLabel:SetText(Strings["channel_header"]);
	
	left = left + channelHeaderLabel:GetWidth() + Constants.ColumnSpacing;
	
	self.ChannelNameLabel = CreateLabel(self, false);
	self.ChannelNameLabel:SetPosition(left, top);
	self.ChannelNameLabel:SetWidth(Constants.Sizes.LabelWidth);
	self.ChannelNameLabel:SetText("");
	
	top = top + channelHeaderLabel:GetHeight() + Constants.LineSpacing;
	left = Constants.Padding.Left;
	
	local patternsSetLabel = CreateLabel(self, false);
	patternsSetLabel:SetPosition(left, top);
	patternsSetLabel:SetWidth(Constants.Sizes.LabelWidth);
	patternsSetLabel:SetText(Strings["pattern_list"]);
	
	left = left + patternsSetLabel:GetWidth() + Constants.ColumnSpacing;
	
	self.PatternListDropdown = CreateDropdown(self, left, top, Constants.Sizes.Dropdown.Width, Constants.Sizes.Dropdown.Height, Constants.Sizes.Dropdown.Open);
	self.PatternListDropdown:SetAutoSort(true, 4);
	self.PatternListDropdown.SelectedIndexChanged = function(sender, args)
		self.PatternsSaved = true;
		self.PatternListWindow:Hide();
		self:SetPatternButtonStatus();
		self:SetSaveButtonsStatus(true);
	end
	
	top = top + patternsSetLabel:GetHeight() + Constants.LineSpacing;
	left = self.PatternListDropdown:GetLeft();

	self.AddPatternsButton = CreateButton(self, 0);
	self.AddPatternsButton:SetPosition(left, top);
	self.AddPatternsButton:SetText(Strings["add"]);
	self.AddPatternsButton.Click = function(sender, args)
		if self.CopyMode then
			local listId = self.PatternListDropdown:GetSelectedValue();
			local blockedPatterns = self.ChangedPatternLists[listId] or Settings.PatternLists[listId];
			self.PatternListWindow:Show(tostring(self.NextPatternListId), blockedPatterns, "", nil);
		else
			self.PatternListWindow:Show(tostring(self.NextPatternListId), {}, "", nil);
		end
	end
	
	left = left + self.AddPatternsButton:GetWidth() + Constants.ColumnSpacing;
	
	self.EditPatternsButton = CreateButton(self, 0);
	self.EditPatternsButton:SetPosition(left, top);
	self.EditPatternsButton:SetText(Strings["edit"])
	self.EditPatternsButton.Click = function(sender, args)
		local listId = self.PatternListDropdown:GetSelectedValue();
		local blockedPatterns = self.ChangedPatternLists[listId] or Settings.PatternLists[listId];
		self.PatternListWindow:Show(listId, blockedPatterns, blockedPatterns.Name, self.ChannelType);
	end
	
	left = left + self.EditPatternsButton:GetWidth() + Constants.ColumnSpacing;
	
	self.RemovePatternsButton = CreateButton(self, 0);
	self.RemovePatternsButton:SetPosition(left, top);
	self.RemovePatternsButton:SetText(Strings["remove"])
	self.RemovePatternsButton.Click = function(sender, args)
		local listId = self.PatternListDropdown:GetSelectedValue();
		if listId ~= nil then
			local listInUse = false;
			for key, data in pairs(Settings.ChannelData) do
				if key ~= self.ChannelType and listId == data.CurrentPatternList then
					ShowBabbleMessage(Strings["remove_not_allowed"], 2);
					listInUse = true;
				end
			end
			if not listInUse then
				self.PatternsSaved = true;
				self.ChangedPatternLists[listId] = { Remove = true };
				self.PatternListDropdown:RemoveSelectedItem();
				self:SetPatternButtonStatus();
				self:SetSaveButtonsStatus(true);
			end
		end
	end
	
	top = top + patternsSetLabel:GetHeight() + Constants.LineSpacing;
	left = Constants.Padding.Left;
	
	local nameListLabel = CreateLabel(self, false);
	nameListLabel:SetPosition(left, top);
	nameListLabel:SetWidth(Constants.Sizes.LabelWidth);
	nameListLabel:SetText(Strings["named_players"]);
	
	left = left + nameListLabel:GetWidth() + Constants.ColumnSpacing;
	
	self.NameListDropdown = CreateDropdown(self, left, top, Constants.Sizes.Dropdown.Width, Constants.Sizes.Dropdown.Height, Constants.Sizes.Dropdown.Open);
	self.NameListDropdown:SetAutoSort(true, 3);
	self.NameListDropdown.SelectedIndexChanged = function(sender, args)
		self.NamesSaved = true;
		self.NameListWindow:Hide();
		self:SetNameButtonStatus();
		self:SetSaveButtonsStatus(true);
	end
	
	top = top + nameListLabel:GetHeight() + Constants.LineSpacing;
	left = self.NameListDropdown:GetLeft();
	
	self.AddNameListButton = CreateButton(self, 0);
	self.AddNameListButton:SetPosition(left, top);
	self.AddNameListButton:SetText(Strings["add"]);
	self.AddNameListButton.Click = function(sender, args)
		if self.CopyMode and self.NameListDropdown:GetSelectedValue() ~= "0" then
			local listId = self.NameListDropdown:GetSelectedValue();
			local names = self.ChangedNameLists[listId] or Settings.NameLists[listId] or {};
			self.NameListWindow:Show(tostring(self.NextNameListId), names, "", nil);
		else
			self.NameListWindow:Show(tostring(self.NextNameListId), {}, "", nil);
		end
	end
	
	left = left + self.AddNameListButton:GetWidth() + Constants.ColumnSpacing;
	
	self.EditNameListButton = CreateButton(self, 0);
	self.EditNameListButton:SetPosition(left, top);
	self.EditNameListButton:SetText(Strings["edit"])
	if self.NextNameListId == 1 then
		self.EditNameListButton:SetEnabled(false);
	end
	self.EditNameListButton.Click = function(sender, args)
		local listId = self.NameListDropdown:GetSelectedValue();
		local names = self.ChangedNameLists[listId] or Settings.NameLists[listId] or {};
		self.NameListWindow:Show(listId, names, names.Name, self.ChannelType);
	end
	
	left = left + self.EditNameListButton:GetWidth() + Constants.ColumnSpacing;
	
	self.RemoveNameListButton = CreateButton(self, 0);
	self.RemoveNameListButton:SetPosition(left, top);
	self.RemoveNameListButton:SetText(Strings["remove"]);
	self.RemoveNameListButton:SetEnabled(self.NameListDropdown:GetSelectedValue() ~= "0");
	self.RemoveNameListButton.Click = function(sender, args)
		local listId = self.NameListDropdown:GetSelectedValue();
		if listId ~= nil then
			local listInUse = false;
			for key, data in pairs(Settings.ChannelData) do
				if key ~= self.ChannelType then
					if listId == data.CurrentNameList then
						ShowBabbleMessage(Strings["remove_not_allowed"], 2);
						listInUse = true;
					end
				end
			end
			if not listInUse then
				self.NamesSaved = true;
				self.ChangedNameLists[listId] = { Remove = true };
				self.NameListDropdown:RemoveSelectedItem();
				self:SetNameButtonStatus();
				self:SetSaveButtonsStatus(true);
			end
		end
	end
	
	top = top + self.AddNameListButton:GetHeight() + Constants.LineSpacing;
	left = self.AddNameListButton:GetLeft();
	
	local copyLabel = CreateLabel(self, false);
	copyLabel:SetPosition(left, top);
	copyLabel:SetWidth(Constants.MinimumSize.Width - Constants.Padding.Left - Constants.Padding.Right);
	copyLabel:SetFont(Globals.Fonts.SmallLabel);
	copyLabel:SetText(Strings["copy_list"]);
	
	top = top + copyLabel:GetHeight() + Constants.LineSpacing;
	left = Constants.Padding.Left;
	
	self.StandardCheckBox = CreateCheckBox(self, left, top, Constants.Sizes.CheckBox, Globals.Sizes.CheckBoxOnly.Height, Strings["route_to_standard"]);
	self.StandardCheckBox.CheckedChanged = function(sender, args) 
		if self.StandardCheckBox.SetChanged == nil then
			self.StandardCheckBoxChanged = true;
			self:SetSaveButtonsStatus(true);
		else
			self.StandardCheckBox.SetChanged = nil;
		end
	end
	
	top = top + self.StandardCheckBox:GetHeight() + Constants.LineSpacing;
	
	self.ExtraCheckBox = CreateCheckBox(self, left, top, Constants.Sizes.CheckBox, Globals.Sizes.CheckBoxOnly.Height, "");
	self.ExtraCheckBox:SetVisible(false);
	self.ExtraCheckBox.CheckedChanged = function(sender, args) 
		if self.ExtraCheckBox.SetChanged == nil then
			self.ExtraCheckBoxChanged = true;
			self:SetSaveButtonsStatus(true);
		else
			self.ExtraCheckBox.SetChanged = nil;
		end
	end
	
	self.WarningLabel = CreateLabel(self, true);
	self.WarningLabel:SetPosition(left, top);
	self.WarningLabel:SetHeight(self.WarningLabel:GetHeight() * 2);
	self.WarningLabel:SetForeColor(Globals.Colours.Warning);
	self.WarningLabel:SetText(Strings["changes_to_save"]);
	self.WarningLabel:SetVisible(false);
	
	local confirmWindow = ConfirmationWindow();
	confirmWindow:SetZOrder(self:GetZOrder() + 10);
	confirmWindow.ButtonClick = function (sender,args)
		self.CancelPressed = true;
		self:Hide();
	end
	
	self.CancelButton = CreateButton(self);
	self.CancelButton:SetText(Strings["cancel"]);
	self.CancelButton.MouseClick = function(sender, args)
		self:Hide();
	end
	
	self.ApplyButton = CreateButton(self);
	self.ApplyButton:SetText(Strings["apply"]);
	self.ApplyButton.MouseClick = function(sender, args)
		if args.Button == Turbine.UI.MouseButton.Left then
			self:SaveSettings(false);
		end
	end
	
	self.SaveButton = CreateButton(self);
	self.SaveButton:SetText(Strings["ok"]);
	self.SaveButton.MouseClick = function(sender, args)
		if args.Button == Turbine.UI.MouseButton.Left then
			self.SavePressed = true;
			if not self:SaveSettings(true) then
				self.SavePressed = false;
			end
		end
	end
	
	self.KeyDown = function (sender, args)
		if args.Shift then
			self:SetAddButtonStatus(true);
		end
		KeyDownActions(self, args);
	end
	
	self.KeyUp = function (sender, args)
		if args.Shift then
			self:SetAddButtonStatus(false);
		end
	end
	
	self.Closing = function(sender, args)
		if not (self.CancelPressed or self.SavePressed) and self:HasChanges() then
			args.Cancel = true;
			confirmWindow:ShowWindow(Strings["settings"], Strings["changes_made"], false, true);
		else
			self.CancelPressed = false;
			self.SavePressed = false;
		end
	end
	
	self.VisibleChanged = function(sender, args)
		if not self:IsVisible() then
			if not self.UIHidden and not self.ReopenOnToggle then
				self.PatternListWindow:SetVisible(false);
				self.NameListWindow:SetVisible(false);
			end
		else
			self:CheckWindowPosition();
		end
	end

	self.Saved = function() end;
	
	EnableCheckBox(self.StandardCheckBox, Settings.EnableChatWindow);
	
	self:LayoutControls();
	self:SetResizer(Constants.MinimumSize.Width, Constants.MinimumSize.Height);
end

function SettingsWindow:LayoutControls()
	local dropdownWidth = self:GetWidth() - Constants.Padding.Right - self.PatternListDropdown:GetLeft();
	self.PatternListDropdown:SetWidth(dropdownWidth);
	self.NameListDropdown:SetWidth(dropdownWidth);
	
	self.WarningLabel:SetWidth(self:GetWidth() - Constants.Padding.Right - Constants.Padding.Left);
	
	local buttonsWidth = self.CancelButton:GetWidth() + Constants.ColumnSpacing + self.ApplyButton:GetWidth() + Constants.ColumnSpacing + self.SaveButton:GetWidth();
	
	self.CancelButton:SetPosition((self:GetWidth() - buttonsWidth) / 2, self:GetHeight() - Constants.Padding.Bottom - self.CancelButton:GetHeight());
	self.ApplyButton:SetPosition(self.CancelButton:GetLeft() + self.CancelButton:GetWidth() + Constants.ColumnSpacing, self.CancelButton:GetTop());
	self.SaveButton:SetPosition(self.ApplyButton:GetLeft() + self.ApplyButton:GetWidth() + Constants.ColumnSpacing, self.CancelButton:GetTop());
end

function SettingsWindow:Show(channelType)
	self.ChannelType = channelType;
	self.CancelPressed = false;
	self.SavePressed = false;
	
	self.ChannelNameLabel:SetText(Strings["channel_" .. channelType]);
	if channelType == ChannelTypes[Turbine.ChatType.Say] then
		self.ExtraCheckBox:SetText(" " .. Strings["settings_npc_say"]);
		self.ExtraCheckBox:SetVisible(true);
		self.WarningLabel:SetTop(self.ExtraCheckBox:GetTop() + self.ExtraCheckBox:GetHeight() + Constants.LineSpacing);
	else
		self.ExtraCheckBox:SetVisible(false);
		self.WarningLabel:SetTop(self.ExtraCheckBox:GetTop());
	end
	
	self:PopulateSettings();
	self.PatternListWindow:SetVisible(false);
	self.NameListWindow:SetVisible(false);
	self:SetVisible(true);
	self:Activate();
end

function SettingsWindow:Hide()
	self:Close();
end

function SettingsWindow:PopulateSettings()
	local channelData = Settings.ChannelData[self.ChannelType];

	self:ResetSettings();
	
	self.PatternListDropdown:ClearItems();
	self.PatternListDropdown:AddItem(Strings["select_list"], "");
	self.PatternListDropdown:AddItem(Strings["allow_all_list"], "-1");
	self.PatternListDropdown:AddItem(Strings["block_all_list"], "-2");
	for key, value in pairs(Settings.PatternLists) do
		self.PatternListDropdown:AddItem(Settings.PatternLists[key].Name, key);
	end
	self.PatternListDropdown:SetSelectedValue(channelData.CurrentPatternList or "", false);
	self:SetPatternButtonStatus();
	
	self.NameListDropdown:ClearItems();
	self.NameListDropdown:AddItem(Strings["select_list"], "");
	self.NameListDropdown:AddItem(Strings["no_name_list"], "0");
	for key, value in pairs(Settings.NameLists) do
		self.NameListDropdown:AddItem(Settings.NameLists[key].Name, key);
	end
	self.NameListDropdown:SetSelectedValue(channelData.CurrentNameList or "", false);
	self:SetNameButtonStatus();
	
	if self.StandardCheckBox:IsChecked() ~= channelData.RouteToStandard then
		self.StandardCheckBox.SetChanged = false;
		self.StandardCheckBox:SetChecked(channelData.RouteToStandard);
	end
	
	if self.ChannelType == ChannelTypes[Turbine.ChatType.Say] and self.ExtraCheckBox:IsChecked() ~= channelData.BlockNPC then
		self.ExtraCheckBox.SetChanged = false;
		self.ExtraCheckBox:SetChecked(channelData.BlockNPC);
	end
end

function SettingsWindow:ResetSettings()
	self.ChangedPatternLists = {};
	self.ChangedNameLists = {};
	self.NextPatternListId = tonumber(Settings.NextPatternListId);
	self.NextNameListId = tonumber(Settings.NextNameListId);
	self.PatternsSaved = false;
	self.NamesSaved = false;
	self.StandardCheckBoxChanged = false;
	self.ExtraCheckBoxChanged = false;
	
	self.WarningLabel:SetVisible(false);
	self:SetSaveButtonsStatus(false);
end

function SettingsWindow:SetSaveButtonsStatus(enabled)
	self.ApplyButton:SetEnabled(enabled);
	self.SaveButton:SetEnabled(enabled);
	self.WarningLabel:SetVisible(enabled);
end

function SettingsWindow:SetPatternButtonStatus()
	local selectedValue = self.PatternListDropdown:GetSelectedValue();
	local enabled = selectedValue ~= "" and selectedValue ~= "-1" and selectedValue ~= "-2";
	self.EditPatternsButton:SetEnabled(enabled);
	self.RemovePatternsButton:SetEnabled(enabled and self.PatternListDropdown:GetItemCount() > 2);
end

function SettingsWindow:SetNameButtonStatus()
	local enabled = self.NameListDropdown:GetSelectedValue() ~= "" and self.NameListDropdown:GetSelectedValue() ~= "0";
	self.EditNameListButton:SetEnabled(enabled);
	self.RemoveNameListButton:SetEnabled(enabled);
end

function SettingsWindow:SetAddButtonStatus(copyMode)
	self.CopyMode = copyMode;
	
	local text = (copyMode and Strings["copy"]) or Strings["add"];
	
	self.AddPatternsButton:SetText(text);
	if self.NameListDropdown:GetSelectedValue() ~= "0" then
		self.AddNameListButton:SetText(text);
	end
end

function SettingsWindow:EnableStandardButton(enabled)
	EnableCheckBox(self.StandardCheckBox, enabled);
end

function SettingsWindow:SaveSettings(closeWindow)
	local channelData = Settings.ChannelData[self.ChannelType];
	
	if self.PatternListWindow:IsVisible() or self.NameListWindow:IsVisible() then
		ShowBabbleMessage(Strings["close_windows"], 2);
		return false;
	end
	
	local currentPatternList = self.PatternListDropdown:GetSelectedValue();
	if currentPatternList == "" then
		ShowBabbleMessage(Strings["select_pattern_list"], 2);
		return false;
	end
	
	local currentNameList = self.NameListDropdown:GetSelectedValue();
	if currentNameList == "" then
		ShowBabbleMessage(Strings["select_name_list"], 2);
		return false;
	end
	
	if self.PatternsSaved then
		for key, value in pairs(self.ChangedPatternLists) do
			if value.Remove then
				Settings.PatternLists[key] = nil;
			else
				Settings.PatternLists[key] = value;
			end
		end
	end
	channelData.CurrentPatternList = currentPatternList;
	Settings.NextPatternListId = tostring(self.NextPatternListId);
	
	if self.NamesSaved then
		for key, value in pairs(self.ChangedNameLists) do
			if value.Remove then
				Settings.NameLists[key] = nil;
			else
				Settings.NameLists[key] = value;
			end
		end
	end
	channelData.CurrentNameList = currentNameList;
	Settings.NextNameListId = tostring(self.NextNameListId);
	
	channelData.RouteToStandard = self.StandardCheckBox:IsChecked();
	
	if self.ChannelType == ChannelTypes[Turbine.ChatType.Say] then
		channelData.BlockNPC = self.ExtraCheckBox:IsChecked();
	end

	self.Saved(self, {});
	
	if closeWindow then
		self:Hide();
	else
		self:ResetSettings();
	end
	
	return true;
end

function SettingsWindow:HasChanges()
	return (self.PatternsSaved or self.NamesSaved or self.StandardCheckBoxChanged or self.ExtraCheckBoxChanged);
end

function SettingsWindow:ChildWindowsOpen()
	return self.PatternListWindow:IsVisible() or self.NameListWindow:IsVisible()
end

function SettingsWindow:SavePositionSettings()
	Settings.SettingsWindow.Position.Top = tostring(self:GetTop());
	Settings.SettingsWindow.Position.Left = tostring(self:GetLeft());
	Settings.SettingsWindow.Size.Width = tostring(self:GetWidth());
	Settings.SettingsWindow.Size.Height = tostring(self:GetHeight());
	
	self.PatternListWindow:SavePositionSettings();
	self.NameListWindow:SavePositionSettings();
end