import "Simbo.Babble.WindowBasic";

StatusButton = class(WindowBasic);

local Constants =
{
	Size =
	{
		Width = 100,
		Height = 24
	},
	Buttons =
	{
		Main =
		{
			Top = 0,
			Left = 0,
			Width = 60,
			Height = 24,
			Normal = "Simbo/Babble/Icons/ButtonMain.tga",
			Hover = "Simbo/Babble/Icons/ButtonMainHover.tga",
			ShortCutImages = {}
		},
		On =
		{
			Top = 0,
			Left = 60,
			Width = 20,
			Height = 24,
			Normal = "Simbo/Babble/Icons/ButtonStatusOn.tga",
			Hover = "Simbo/Babble/Icons/ButtonStatusOnHover.tga",
			ShortCutImages = {}
		},
		Off =
		{
			Top = 0,
			Left = 60,
			Width = 20,
			Height = 24,
			Normal = "Simbo/Babble/Icons/ButtonStatusOff.tga",
			Hover = "Simbo/Babble/Icons/ButtonStatusOffHover.tga",
			ShortCutImages = {}
		},
		Chat =
		{
			Top = 0,
			Left = 80,
			Width = 20,
			Height = 24,
			Normal = "Simbo/Babble/Icons/ButtonChat.tga",
			Hover = "Simbo/Babble/Icons/ButtonChatHover.tga",
			ShortCutImages = {}
		}
	}
}

function StatusButton:Constructor()
	self.AlwaysVisible = true;
	WindowBasic.Constructor(self);

	self:SetPosition(tonumber(Settings.StatusButton.Position.Left) or 100, tonumber(Settings.StatusButton.Position.Top) or 100);
	self:SetSize(Constants.Size.Width, Constants.Size.Height);
	self:SetOpacity(Globals.Opacity);
	self:SetVisible(true);
	
	self.ButtonMain = self:CreateIconButton("Main");
	self.ButtonMain.Clicked = function()
		StatusWindow:Toggle();
	end
	
	self.ButtonStatus = self:CreateIconButton("On");
	self.ButtonStatus.Clicked = function()
		EnableBabble(self.ButtonStatus.Icon == "Off", true);
	end	
	
	self.ButtonChat = self:CreateIconButton("Chat");
	self.ButtonChat.Clicked = function()
		ShowChatWindow(not ChatWindow:IsVisible(), true);
	end	
	
	self.VisibleChanged = function(sender, args)
		if self:IsVisible() then
			self:CheckWindowPosition();
		end
	end
	
	self:ShowChatButton(Settings.EnableChatWindow);
end

function StatusButton:CreateIconButton(iconName)
	local iconData = Constants.Buttons[iconName];
	
	local button = Turbine.UI.Control();
	button:SetParent(self);
	button:SetPosition(iconData.Left, iconData.Top);
	button:SetSize(iconData.Width, iconData.Height);
	button:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend);
	--button:SetBackground(iconData.Normal);
	Window_SetIconImage(iconData.ShortCutImages, button, "Normal", iconData);
	button.Icon = iconName;
	button.Clicked = function() end;
	
	button.MouseEnter = function(sender,args)
		local iconData = Constants.Buttons[sender.Icon];
		--sender:SetBackground(Constants.Buttons[sender.Icon].Hover);
		Window_SetIconImage(iconData.ShortCutImages, sender, "Hover", iconData);
	end
	
	button.MouseLeave = function(sender,args)
		local iconData = Constants.Buttons[sender.Icon];
		--sender:SetBackground(Constants.Buttons[sender.Icon].Normal);
		Window_SetIconImage(iconData.ShortCutImages, sender, "Normal", iconData);
	end
	
	button.MouseDown = function(sender, args)
		if args.Button == Turbine.UI.MouseButton.Left then
			sender.DragStartX = args.X;
			sender.DragStartY = args.Y;
			sender.Dragging = true;
			sender.Dragged = false;
		end
	end
	
	button.MouseUp = function(sender, args) 
		if args.Button == Turbine.UI.MouseButton.Left then			
			sender.Dragging = false;
			if not sender.Dragged then
				sender:Clicked();
			end
		end
	end
	
	button.MouseMove = function(sender,args)
		if sender.Dragging then
			local left, top = self:GetPosition();
			self:SetPosition(left + (args.X - sender.DragStartX), top + args.Y - sender.DragStartY);
			sender.Dragged = true;

			if self:GetLeft() > Turbine.UI.Display.GetWidth() - Constants.Size.Width then
				self:SetLeft(Turbine.UI.Display.GetWidth() - Constants.Size.Width);				
			end
			if self:GetLeft() < 0 then
				self:SetLeft(0);				
			end			
			if self:GetTop() > Turbine.UI.Display.GetHeight() - Constants.Size.Height then
				self:SetTop(Turbine.UI.Display.GetHeight() - Constants.Size.Height);				
			end
			if self:GetTop() < 0 then
				self:SetTop(0);				
			end			
		end
	end
	
	return button;
end

function StatusButton:SetButtonState(enabled)
	if enabled then
		self.ButtonStatus.Icon = "On";
	else
		self.ButtonStatus.Icon = "Off";
	end
	local iconData = Constants.Buttons[self.ButtonStatus.Icon];
	Window_SetIconImage(iconData.ShortCutImages, self.ButtonStatus, "Normal", iconData);
	--self.ButtonStatus:SetBackground(Constants.Buttons[self.ButtonStatus.Icon].Normal);
end

function StatusButton:ShowChatButton(visible)
	self.ButtonChat:SetVisible(visible);
end

function StatusButton:SavePositionSettings()
	Settings.StatusButton.Position.Left = tostring(self:GetLeft());
	Settings.StatusButton.Position.Top = tostring(self:GetTop());		
end
