import "Simbo.Babble.WindowLotro";
import "Simbo.Babble.ConfirmationWindow";
import "Simbo.Babble.SettingsWindow";
import "Simbo.Babble.ColoursWindow";

StatusWindow = class(WindowLotro);

local Constants =
{
	FixedSize =
	{
		Width = 240,
		Height = 390
	},
	Padding =
	{
		Top = 40,
		Left = 20,
		Bottom = 20,
		Right = 20
	},
	Sizes =
	{
		LabelWidth = 110,
		CheckBoxNarrow = 110,
		CheckBoxWide = 190,
		IndicatorWidth = 30
	},
	ScrollBars =
	{
		Size = { Width = 320, Height = 11 } 
	},
	LineSpacing = 2,
	ColumnSpacing = 8
}

function StatusWindow:Constructor()
	WindowLotro.Constructor(self);
	
	self.SettingsCheckBox = nil;
	self.Buttons = {};
	
	self:SetText(Strings["babble_header"]);
	self:SetPosition(tonumber(Settings.StatusWindow.Position.Left) or 100, tonumber(Settings.StatusWindow.Position.Top) or 100);
	self:SetSize(Constants.FixedSize.Width, Constants.FixedSize.Height);
	self:SetOpacity(Globals.Opacity);
	
	self.SettingsWindow = SettingsWindow();
	self.SettingsWindow.Saved = function (sender, args)
		local channelType;
		for _, channelType in pairs(ChannelTypes) do
			BabbleHelpers[channelType]:SelectPatternList(Settings.ChannelData[channelType].CurrentPatternList);
			BabbleHelpers[channelType]:SelectNameList(Settings.ChannelData[channelType].CurrentNameList);
		end
		SaveBabbleSettings(false);

		self:EnableChannel(self.SettingsCheckBox, true, false, false);
	end
	
	self.ColoursWindow = ColoursWindow();
	self.ColoursWindow:SetMode(Settings.EnableChatWindow);
	self.ColoursWindow.Saved = function (sender, args)
		SaveBabbleSettings(false);
	end
	
	local top = Constants.Padding.Top;
	local left = Constants.Padding.Left;
	
    self.BabbleEnabledCheckBox = CreateCheckBox(self, left, top, Constants.Sizes.CheckBoxWide, Globals.Sizes.CheckBoxOnly.Height, Strings["enable_babble"]);
    self.BabbleEnabledCheckBox:SetCheckAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
    self.BabbleEnabledCheckBox.UpdateStatus = true;
	self.BabbleEnabledCheckBox.CheckedChanged = function(sender, args)
		local isChecked = self.BabbleEnabledCheckBox:IsChecked();
		if self.BabbleEnabledCheckBox.UpdateStatus then
			EnableBabble(isChecked);
		else
			self.BabbleEnabledCheckBox.UpdateStatus = true;
		end
		EnableCheckBox(self.ChatWindowCheckbox, isChecked);
		
		for _, channelType in ipairs(SortedChannels) do
			self.Buttons[channelType]:SetEnabled(isChecked);
			EnableCheckBox(self.Buttons[channelType].CheckBox, isChecked and Settings.ChannelData[channelType].CurrentPatternList ~= nil);
		end
	end
	
	top = top + self.BabbleEnabledCheckBox:GetHeight() + Constants.LineSpacing;
	
	self.ChatWindowCheckbox = CreateCheckBox(self, left, top, Constants.Sizes.CheckBoxWide, Globals.Sizes.CheckBoxOnly.Height, Strings["babble_chatwindow"]);
    self.ChatWindowCheckbox:SetCheckAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	self.ChatWindowCheckbox:SetChecked(Settings.EnableChatWindow);
	self.ChatWindowCheckbox.CheckedChanged = function(sender, args)
		local isChecked = self.ChatWindowCheckbox:IsChecked();
		self:EnableControlsForWindow(isChecked);
		EnableChatWindow(isChecked);
		self.SettingsWindow:EnableStandardButton(isChecked);
		self.ColoursWindow:SetMode(isChecked);
	end
	
	top = top + self.ChatWindowCheckbox:GetHeight();
	left = left + Globals.Sizes.CheckBoxOnly.Width;
	
	self.ShowChatWindowCheckbox = CreateCheckBox(self, left, top, Constants.Sizes.CheckBoxWide, Globals.Sizes.CheckBoxOnly.Height, Strings["babble_showchatwindow"]);
    self.ShowChatWindowCheckbox:SetCheckAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	self.ShowChatWindowCheckbox:SetFont(Globals.Fonts.SmallLabel);
	self.ShowChatWindowCheckbox:SetChecked(Settings.ShowChatWindow);
	self.ShowChatWindowCheckbox.CheckedChanged = function(sender, args)
		if self.ShowChatWindowCheckbox.ManualSet == nil then
			ShowChatWindow(self.ShowChatWindowCheckbox:IsChecked(), false);
		else
			self.ShowChatWindowCheckbox.ManualSet = nil;
		end
	end
	
	top = top + self.ShowChatWindowCheckbox:GetHeight();
	
	self.ShowTimestampCheckbox = CreateCheckBox(self, left, top, Constants.Sizes.CheckBoxWide, Globals.Sizes.CheckBoxOnly.Height, Strings["show_timestamp"]);
    self.ShowTimestampCheckbox:SetCheckAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	self.ShowTimestampCheckbox:SetFont(Globals.Fonts.SmallLabel);
	self.ShowTimestampCheckbox:SetChecked(Settings.ShowTimestamp);
	self.ShowTimestampCheckbox.CheckedChanged = function(sender, args)
		Settings.ShowTimestamp = self.ShowTimestampCheckbox:IsChecked();
	end
	
	top = top + self.ShowTimestampCheckbox:GetHeight();

	self.OpacityLabel = CreateLabel(self, false);
	self.OpacityLabel:SetWidth(self:GetWidth() - left - Constants.Padding.Right - Constants.Sizes.IndicatorWidth);
	self.OpacityLabel:SetPosition(left, top);
	self.OpacityLabel:SetFont(Globals.Fonts.SmallLabel);
	self.OpacityLabel:SetText(Strings["window_opacity"] .. ": ");
	
	top = top + self.OpacityLabel:GetHeight();
	
	local opacity = tonumber(Settings.ChatWindowOpacity);
	local scrollObjects = self:CreateScrollBar(left, top, opacity, 50);
	self.OpacityScrollBar = scrollObjects.ScrollBar;
	self.OpacityIndicator = scrollObjects.Indicator;
	self.OpacityIndicator:SetPosition(left + self.OpacityLabel:GetWidth(), self.OpacityLabel:GetTop());
	self.OpacityScrollBar.ValueChanged = function(sender,args)
		local newValue = sender:GetValue() / 100;
		self.OpacityIndicator:SetText(math:FormatNumber(newValue));
		ChatWindow:SetOpacity(newValue);
		Settings.ChatWindowOpacity = tostring(sender:GetValue());
	end	
	
	self:EnableControlsForWindow(Settings.EnableChatWindow);
	
	top = top + self.OpacityScrollBar:GetHeight() + Constants.LineSpacing;
	left = Constants.Padding.Left;
	
	self.EnableStatusButton = CreateCheckBox(self, left, top, Constants.Sizes.CheckBoxWide, Globals.Sizes.CheckBoxOnly.Height, Strings["enable_status_button"]);
    self.EnableStatusButton:SetCheckAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	self.EnableStatusButton:SetChecked(Settings.EnableBabbleButton);
	self.EnableStatusButton.CheckedChanged = function(sender, args)
		local isChecked = self.EnableStatusButton:IsChecked();
		StatusButton:SetVisible(self.EnableStatusButton:IsChecked());
		Settings.EnableBabbleButton = isChecked;
	end
	
	top = top + self.EnableStatusButton:GetHeight() + Constants.LineSpacing;
	
	local headerLabel = CreateLabel(self, false);
	headerLabel:SetWidth(Constants.Sizes.LabelWidth);
	headerLabel:SetPosition(left, top);
	headerLabel:SetText(string.upper(Strings["channels"]));
	
	top = top + headerLabel:GetHeight() + Constants.LineSpacing;
	
	for _, channelType in ipairs(SortedChannels) do
		local babbleHelper = BabbleHelpers[channelType];
		local channelData = Settings.ChannelData[channelType];
		local enabled = babbleHelper:GetStatus();
		
		local channelName = Strings["channel_" .. channelType];
		local checkBox = CreateCheckBox(self, left, top, Constants.Sizes.CheckBoxNarrow, Globals.Sizes.CheckBoxOnly.Height, channelName);
		checkBox:SetChecked(enabled);
		checkBox.CheckedChanged = function(sender, args)
			self:EnableChannel(sender, nil, sender:IsChecked(), true);
			if sender:IsChecked() then
				ShowBabbleMessage(Strings["babble_filtering"]:format(channelName));
			else
				ShowBabbleMessage(Strings["babble_not_filtering"]:format(channelName));
			end
		end

		checkBox.ChannelType = channelType;
		self:EnableChannel(checkBox, channelData.CurrentPatternList ~= nil, enabled, false);
		
		left = left + checkBox:GetWidth();
		
		local button = CreateButton(self);
		button:SetPosition(left, top);
		button:SetText(Strings["edit"]);
		button.CheckBox = checkBox;
		button.MouseClick = function(sender, args)
			if args.Button == Turbine.UI.MouseButton.Left then
				if (self.SettingsWindow:IsVisible() and self.SettingsWindow:HasChanges()) or self.SettingsWindow:ChildWindowsOpen() then
					if sender.CheckBox.ChannelType ~= self.SettingsCheckBox.ChannelType then
						self.ConfirmWindow.CheckBox = sender.CheckBox;
						self.ConfirmWindow:ShowWindow(Strings["settings"], Strings["settings_open"], false, true);
					else
						self.SettingsWindow:Activate();
					end
				else
					self.SettingsCheckBox = sender.CheckBox;
					self.SettingsWindow:Show(sender.CheckBox.ChannelType);
				end
			end
		end
		self.Buttons[channelType] = button;
		
		top = top + checkBox:GetHeight() + Constants.LineSpacing;
		left = Constants.Padding.Left;
	end
	
	top = top + Constants.LineSpacing;
	
	local buttonAdjuster = UIOptions["wide_status_button"] and 2 or 1;
	
	self.ColoursButton = CreateButton(self, buttonAdjuster);
	self.ColoursButton:SetPosition((self:GetWidth() - self.ColoursButton:GetWidth()) / 2, top);
	self.ColoursButton:SetText(Strings["edit_colours"]);
	self.ColoursButton.Click = function(sender, args)
		self.ColoursWindow:Show();
	end
	
	self.ConfirmWindow = ConfirmationWindow();
	self.ConfirmWindow:SetZOrder(self:GetZOrder() + 10);
	self.ConfirmWindow.ButtonClick = function (sender,args)
		self.SettingsCheckBox = self.ConfirmWindow.CheckBox;
		self.SettingsWindow:Show(self.ConfirmWindow.CheckBox.ChannelType);
		self.ConfirmWindow.CheckBox = nil;
	end
end

function StatusWindow:Show()
	if not self:IsVisible() then
		ShowChildWindow(self.SettingsWindow);
		ShowChildWindow(self.SettingsWindow.NameListWindow);
		ShowChildWindow(self.SettingsWindow.PatternListWindow);
		ShowChildWindow(self.ColoursWindow);
		self:SetVisible(true);
	end
	self:Activate();
end

function StatusWindow:Hide()
	if self:IsVisible() then
		HideChildWindow(self.SettingsWindow.NameListWindow);
		HideChildWindow(self.SettingsWindow.PatternListWindow);
		HideChildWindow(self.SettingsWindow);
		HideChildWindow(self.ColoursWindow);
		self:SetVisible(false);
	end
end

function StatusWindow:Toggle()
	if self:IsVisible() then
		self:Hide();
	else
		self:Show();
	end
end

function ShowChildWindow(childWindow)
	if childWindow.ReopenOnToggle then
		childWindow.ReopenOnToggle = false;
		childWindow:SetVisible(true);
	end
end

function HideChildWindow(childWindow)
	if childWindow:IsVisible() then
		childWindow.ReopenOnToggle = true;
		childWindow:SetVisible(false);
	end
end

function StatusWindow:EnableChannel(checkBox, enableCheckBox, channelEnabled, updateHelper)
	local label = checkBox.Label;
	EnableCheckBox(checkBox, enableCheckBox);
	
	if updateHelper then
		local channelType = checkBox.ChannelType;
		local babbleHelper = BabbleHelpers[channelType];
		Settings.ChannelData[channelType].Enabled = channelEnabled;
		babbleHelper:UpdateStatus(channelEnabled);
	end
end

function StatusWindow:EnableBabbleCheckBox(enabled)
	self.BabbleEnabledCheckBox.UpdateStatus = false;
	self.BabbleEnabledCheckBox:SetChecked(enabled);
end

function StatusWindow:EnableControlsForWindow(enabled)
	EnableCheckBox(self.ShowChatWindowCheckbox, enabled);
	EnableCheckBox(self.ShowTimestampCheckbox, enabled);
	EnableLabel(self.OpacityLabel, enabled);
	EnableLabel(self.OpacityIndicator, enabled);
	self.OpacityScrollBar:SetVisible(enabled);
end

function StatusWindow:SavePositionSettings()
	Settings.StatusWindow.Position.Top = tostring(self:GetTop());
	Settings.StatusWindow.Position.Left = tostring(self:GetLeft());
	
	self.SettingsWindow:SavePositionSettings();
	self.ColoursWindow:SavePositionSettings();
end

function StatusWindow:CreateScrollBar(left, top, opacity, minimum)
	local scrollBarWidth = Constants.ScrollBars.Size.Width * (100 - minimum) / 100;
	
	local opacityScroll = Turbine.UI.Lotro.ScrollBar();
	opacityScroll:SetParent(self);
	opacityScroll:SetOrientation(Turbine.UI.Orientation.Horizontal);
	opacityScroll:SetPosition(left, top);
	opacityScroll:SetSize(scrollBarWidth, Constants.ScrollBars.Size.Height);
	opacityScroll:SetMaximum(100);
	opacityScroll:SetMinimum(minimum);
	opacityScroll:SetSmallChange(1);
	opacityScroll:SetLargeChange(5);
	opacityScroll:SetValue(opacity);

	local opacityInd = CreateLabel(self);
	opacityInd:SetFont(Globals.Fonts.SmallLabel);
	opacityInd:SetPosition(left + scrollBarWidth, top - Constants.ScrollBars.Size.Height / 2);
	opacityInd:SetWidth(Constants.Sizes.IndicatorWidth);
	opacityInd:SetText(math:FormatNumber(opacity / 100));
	
	return { ScrollBar = opacityScroll, Indicator = opacityInd }
end