StyledList = class();

local Constants =
{
	Border = 1,
	ScrollBarSize = 10,
	LineHeight = 20,
	Colours =
	{
		NormalOutline = Turbine.UI.Color(1.0, 0.8, 0.65, 0.1),
		HoverOutline = Turbine.UI.Color(0.8, 0.65, 0.1),
		SelectedItem = Turbine.UI.Color(0.90, 0.25, 0.25, 0.25),
		SelectedItemBox = Turbine.UI.Color(0.90, 0.04, 0.04, 0.04)
	}
}

local Styles =
{
	Default = 0,
	Coloured = 1
}

function StyledList:Constructor(parent, style, fixedHeight)
	self.Font = nil;
	self.ForeColour = nil;
	self.SelectedLineIndex = nil;
	self.AutoSort = false;
	self.Style = style or Styles.Default;
	self.FixedHeight = fixedHeight;
		
	self.SelectionList = Turbine.UI.Control();
	self.SelectionList:SetParent(parent);
	
	self.SelectionListBox = Turbine.UI.ListBox();
	self.SelectionListBox:SetParent(self.SelectionList);
	self.SelectionListBox:SetPosition(Constants.Border, Constants.Border);
	self.SelectionListBox:SetOrientation(Turbine.UI.Orientation.Horizontal);
	self.SelectionListBox:SetMaxItemsPerLine(1);
	
	self.SelectedTop = Turbine.UI.Control();
	self.SelectedTop:SetParent(self.SelectionList);
	self.SelectedTop:SetBackColor(Constants.Colours.SelectedItem);
	self.SelectedTop:SetPosition(0, 0);
	self.SelectedTop:SetHeight(1);
	self.SelectedTop:SetVisible(self.Style == Styles.Coloured);
	
	self.SelectedBottom = Turbine.UI.Control();
	self.SelectedBottom:SetParent(self.SelectionList);
	self.SelectedBottom:SetBackColor(Constants.Colours.SelectedItem);
	self.SelectedBottom:SetPosition(0, 0);
	self.SelectedBottom:SetHeight(1);
	self.SelectedBottom:SetVisible(self.Style == Styles.Coloured);
	
	self.SelectionListBox.SelectedIndexChanged = function(sender, args)
		if self.SelectionListBox.ManualReset then
			self.SelectionListBox.ManualReset = nil;
		else
			local previousIndex = self.SelectedLineIndex;
			local index = self.SelectionListBox:GetSelectedIndex();
			self:SelectLine(index);
			
			if self.SelectedLineIndex ~= previousIndex then
			local selectedLine = self.SelectionListBox:GetItem(self.SelectedLineIndex);
				local args = 
				{
					SelectedIndex = self.SelectedLineIndex,
					SelectedText = nil,
					SelectedValue = nil;
				}; 
				if selectedLine ~= nil then
					args.SelectedText = selectedLine:GetText();
					args.SelectedValue = selectedLine.Value;
				end
				self.SelectedIndexChanged(self, args);
			end
		end
	end

	if not self.FixedHeight then
		self.LinesScrollBar = Turbine.UI.Lotro.ScrollBar();
		self.LinesScrollBar:SetOrientation(Turbine.UI.Orientation.Vertical);
		self.LinesScrollBar:SetParent(self.SelectionListBox);
		self.LinesScrollBar:SetZOrder(999);
		self.SelectionListBox:SetVerticalScrollBar(self.LinesScrollBar);
	end
	
	self.SelectedIndexChanged = function() end;
end

function StyledList:SetAutoSort(sort)
	self.AutoSort = sort;
end

function StyledList:Focus()
	self.SelectionList:Focus();
end

function StyledList:SetPosition(left, top)
	self.SelectionList:SetPosition(left, top);
end

function StyledList:SetLeft(left)
	self.SelectionList:SetLeft(left);
end

function StyledList:SetTop(top)
	self.SelectionList:SetTop(top);
end

function StyledList:SetSize(width, height)
	self.SelectionList:SetSize(width, height);

	self.SelectionListBox:SetSize(self.SelectionList:GetWidth() - (Constants.Border * 2), self.SelectionList:GetHeight() - (Constants.Border * 2));
	for i = 1, self.SelectionListBox:GetItemCount() do
		self.SelectionListBox:GetItem(i):SetWidth(self.SelectionListBox:GetWidth());
	end
	
	if not self.FixedHeight then
		self.LinesScrollBar:SetSize(Constants.ScrollBarSize,  self.SelectionListBox:GetHeight());
		self.LinesScrollBar:SetPosition(self.SelectionListBox:GetWidth() - Constants.ScrollBarSize,  0);
	end
	
	self.SelectedTop:SetWidth(self.SelectionListBox:GetWidth());
	self.SelectedBottom:SetWidth(self.SelectionListBox:GetWidth());
end

function StyledList:SetWidth(width)
	self:SetSize(width, self.SelectionList:GetHeight());
end

function StyledList:SetHeight(height)
	self:SetSize(self.SelectionList:GetWidth(), height);
end

function StyledList:GetLeft()
	return self.SelectionList:GetLeft();
end

function StyledList:GetWidth()
	return self.SelectionList:GetWidth();
end

function StyledList:GetHeight()
	return self.SelectionList:GetHeight();
end

function StyledList:GetTop()
	return self.SelectionList:GetTop();
end

function StyledList:SetBorderColor(colour)
	self.SelectionList:SetBackColor(colour);
end

function StyledList:SetBackColor(colour)
	self.SelectionListBox:SetBackColor(colour);	
end

function StyledList:SetOpacity(opacitiy)
	local color = self.SelectionListBox:GetBackColor();
	color.A = opacitiy;
	self:SetBackColor(color);
end

function StyledList:SetForeColor(colour)
	self.ForeColour = colour;
	for i = 1, self.SelectionListBox:GetItemCount(), self.SelectionListBox:GetMaxItemsPerLine() do
		local item = self.SelectionListBox:GetItem(i);
		item:SetForeColor(colour);
		item:SetText(item:GetText());
	end
end

function StyledList:SetFont(font)
	self.Font = font;
	for i = 1, self.SelectionListBox:GetItemCount(), self.SelectionListBox:GetMaxItemsPerLine() do
		local item = self.SelectionListBox:GetItem(i);
		item:SetFont(font);
		item:SetText(item:GetText());
	end
end

function StyledList:SelectLine(index)
	if index % self.SelectionListBox:GetMaxItemsPerLine() == 1 or self.SelectionListBox:GetMaxItemsPerLine() == 1 then
		if self.SelectedLineIndex ~= nil then
			local currentLine = self.SelectionListBox:GetItem(self.SelectedLineIndex);
			currentLine:SetBackColor(self.SelectionListBox:GetBackColor());
		end

		if index > 0 then
			local selectedLine = self.SelectionListBox:GetItem(index);
			if self.Style == Styles.Default then
				selectedLine:SetBackColor(Constants.Colours.SelectedItem);
			else
				self.SelectedTop:SetParent(selectedLine);
				self.SelectedBottom:SetParent(selectedLine);
				self.SelectedBottom:SetTop(selectedLine:GetHeight() - 1);
				selectedLine:SetBackColor(Constants.Colours.SelectedItemBox);
			end
			self.SelectedLineIndex = index;
			if self.SelectionListBox:GetSelectedIndex() ~= index then
				self.SelectionListBox.ManualReset = true;
				self.SelectionListBox:SetSelectedIndex(index);
			end
		else
			self.SelectedLineIndex = nil;
		end
	end
end

function StyledList:GetSelectedLineIndex()
	return self.SelectedLineIndex;
end

function StyledList:GetLineColour(index)
	if index % self.SelectionListBox:GetMaxItemsPerLine() == 1 or self.SelectionListBox:GetMaxItemsPerLine() == 1 then
		local selectedLine = self.SelectionListBox:GetItem(index);
		return selectedLine:GetForeColor();
	end
	
	return nil
end

function StyledList:SetLineColour(index, colour)
	if index % self.SelectionListBox:GetMaxItemsPerLine() == 1 or self.SelectionListBox:GetMaxItemsPerLine() == 1 then
		local selectedLine = self.SelectionListBox:GetItem(index);
		selectedLine:SetForeColor(colour);
	end
end
	
function StyledList:FindInsertIndex(text)
	local itemCount = self.SelectionListBox:GetItemCount();
	local stepCount = self.SelectionListBox:GetMaxItemsPerLine();
	local insertIndex = itemCount + 1;
	for index = 1, itemCount, stepCount do
		local checkText = self.SelectionListBox:GetItem(index):GetText();
		local compare = text:Compare(checkText);
		if compare == 0 then
			insertIndex = nil;
			break;
		elseif compare < 0 then
			insertIndex = index;
			break;
		end
	end

	return insertIndex;
end

function StyledList:AddItem(text, selectNewItem, colour, value)
	local insertIndex = self:GetItemCount() + 1;
	
	if self.AutoSort then
		insertIndex = self:FindInsertIndex(text);
		if insertIndex == nil then
			return;
		end
	end
	
	local label = self:CreateLabelForList(text);
	label.Value = value;
	if colour ~= nil then
		label:SetForeColor(colour);
	end
	
	label.MouseEnter = function(sender, args)
		self:HighlightLabel(sender);
	end
	label.MouseLeave = function(sender, args)
		self:UnhighlightLabel(sender);
	end
	
	self.SelectionListBox:InsertItem(insertIndex, label);
	
	if selectNewItem then
		self:SelectLine(insertIndex);
	else
		if self.SelectedLineIndex ~= nil and self.SelectedLineIndex >= insertIndex then
			self.SelectedLineIndex = self.SelectedLineIndex + self.SelectionListBox:GetMaxItemsPerLine();
		end
	end
end

function StyledList:RemoveSelectedItem()
	if self.SelectedLineIndex > 0 then
		local index = self.SelectedLineIndex;
		for i = 1, self.SelectionListBox:GetMaxItemsPerLine() do
			self:RemoveItem(index, false);
		end
		self.SelectedLineIndex = nil;
	end
end

function StyledList:RemoveItem(index, adjustSelectedIndex)
	self.SelectionListBox:RemoveItemAt(index);
	if adjustSelectedIndex then
		if self.SelectedLineIndex == index then
			self.SelectedLineIndex = nil;
		elseif self.SelectedLineIndex > index then
			self.SelectedLineIndex = self.SelectedLineIndex - self.SelectionListBox:GetMaxItemsPerLine();
		end
	end
end

function StyledList:CreateLabelForList(text)
	local label = Turbine.UI.Label();
	label:SetSize(self.SelectionListBox:GetWidth(), Constants.LineHeight);
	label:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	label:SetMultiline(false);
	if self.ForeColour ~= nil then
		label:SetForeColor(self.ForeColour);
	end
	if self.Font ~= nil then
		label:SetFont(self.Font);
	end
	label:SetText(text);
	
	return label;
end

function StyledList:HighlightLabel(label)
	if self.Style == Styles.Default then
		label:SetOutlineColor(Constants.Colours.HoverOutline);
	else
		label:SetOutlineColor(label:GetForeColor());
	end
	
	label:SetFontStyle(Turbine.UI.FontStyle.Outline);
end

function StyledList:UnhighlightLabel(label)
	label:SetOutlineColor(Constants.Colours.NormalOutline);
	label:SetFontStyle(Turbine.UI.FontStyle.None);
end

function StyledList:ClearItems()
	self.SelectedLineIndex = nil;
	self.SelectionListBox:SetSelectedIndex(0);
	self.SelectionListBox:ClearItems();
end

function StyledList:GetItemCount()
	local itemsPerRow = self.SelectionListBox:GetMaxItemsPerLine();
	if itemsPerRow < 0 then
		itemsPerRow = 1;
	end
	
	return self.SelectionListBox:GetItemCount() / itemsPerRow;
end

function StyledList:GetSelectedItem()
	if self.SelectedLineIndex > 0 then
		return self.SelectionListBox:GetItem(self.SelectedLineIndex):GetText();
	end
	
	return nil;
end

function StyledList:GetSelectedValue()
	if self.SelectedLineIndex > 0 then
		return self.SelectionListBox:GetItem(self.SelectedLineIndex).Value;
	end
	
	return nil;
end

function StyledList:GetAllItems()
	local itemCount = self.SelectionListBox:GetItemCount();
	local items = {};
	for index = 1, itemCount do
		local item = self.SelectionListBox:GetItem(index);
		items[tostring(index)] = item:GetText();
	end
	
	return items;
end