function KeyDownActions(self, args)
	if args.Action == 268435635 then
		if not self.UIHidden then
			self.UIHidden = true;
			self.WasVisible = self:IsVisible();
			if self.WasVisible then
				self:SetVisible(false);					
			end
		else
			if self.WasVisible then
				self:SetVisible(true);
			end
			self.UIHidden = false;
		end
	elseif args.Action == 145 and (self.AlwaysVisible == nil or not self.AlwaysVisible) then
		if self:IsVisible() then
			self.ReopenOnToggle = true;
		end
		self:SetVisible(false);
	end
end

function CreateButton(parent, widthAdjuster)
	local button = Turbine.UI.Lotro.Button();
	button:SetParent(parent);
	if widthAdjuster == 2 then
		button:SetSize(Globals.Sizes.ExtraWideButton.Width, Globals.Sizes.ExtraWideButton.Height);
	elseif widthAdjuster == 1 then
		button:SetSize(Globals.Sizes.WideButton.Width, Globals.Sizes.WideButton.Height);
	elseif widthAdjuster == -1 then
		button:SetSize(Globals.Sizes.NarrowButton.Width, Globals.Sizes.NarrowButton.Height);
	else
		button:SetSize(Globals.Sizes.Button.Width, Globals.Sizes.Button.Height);
	end
	button:SetText("");
	
	return button;
end

function CreateTextBox(parent)
	local textBox = Turbine.UI.Lotro.TextBox();
	textBox:SetParent(parent);
	textBox:SetFont(Globals.Fonts.TextBox);
	textBox:SetSize(Globals.Sizes.TextBox.Width, Globals.Sizes.TextBox.Height);
	textBox:SetForeColor(Globals.Colours.TextBox);
	textBox:SetBackColor(Globals.BackgroundColours.TextBox);
	textBox:SetMultiline(false);
	textBox:SetText("");

	return textBox;
end

function CreateLabel(parent, multiline)
	local label = Turbine.UI.Label();
	label:SetParent(parent);
	label:SetForeColor(Globals.Colours.Label);
	label:SetFont(Globals.Fonts.Label);
	label:SetHeight(Globals.Sizes.Label.Height);
	label:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	label:SetMultiline(multiline or false);
	label:SetText("");

	return label;
end

function CreateCheckBox(parent, left, top, width, height, text)
	local checkBox = Turbine.UI.Lotro.CheckBox();
	checkBox:SetParent(parent);
	if left ~= nil and top ~= nil then
		checkBox:SetPosition(left, top);
	end
	if width ~= nil and height ~= nil then
		checkBox:SetSize(width, height);
	else
		checkBox:SetSize(Globals.Sizes.CheckBoxOnly.Width, Globals.Sizes.CheckBoxOnly.Height);
	end
	checkBox:SetForeColor(Globals.Colours.Label);
	checkBox:SetFont(Globals.Fonts.Label);
	if text ~= nil then
		checkBox:SetText(" " .. text);
	else
		checkBox:SetText("");
	end
	checkBox:SetChecked(false);
	
	return checkBox;
end

function EnableCheckBox(checkBox, enabled)
	EnableControlWithLabel(checkBox, checkBox, enabled);
end

function EnableControlWithLabel(control, label, enabled)
	if enabled ~= nil then
		control:SetEnabled(enabled);
	end
	EnableLabel(label, control:IsEnabled());
end

function EnableLabel(label, enabled)
	if enabled then
		label:SetForeColor(Globals.Colours.Label);
	else
		label:SetForeColor(Globals.Colours.InactiveLabel);
	end
	label:SetText(label:GetText());
end

function CreateDropdown(parent, left, top, width, height, dropdownHeight)
	local dropdown = DropdownList(parent);
	
	dropdown:SetBorderColor(Globals.BorderColours.ListBox);
	dropdown:SetBackColor(Globals.BackgroundColours.ListBox);	
	dropdown:SetForeColor(Globals.Colours.ListBox);	
	dropdown:SetFont(Globals.Fonts.ListBox);
	if width ~= nil and height ~= nil then
		dropdown:SetSize(width, height, dropdownHeight);
	end
	if left ~= nil and top ~= nil then
		dropdown:SetPosition(left, top);
	end

	return dropdown;
end

-- Window Resizer
function Window_SetResizer(self, minimumWidth, minimumHeight, resizerIcon, resizerWidth, resizerHeight, zOrder)
	self.resizeCtrl = Turbine.UI.Control();
	self.resizeCtrl:SetParent(self);
	self.resizeCtrl:SetSize(resizerWidth, resizerHeight);
	self.resizeCtrl:SetStretchMode(1);
	self.resizeCtrl:SetBackColor(Turbine.UI.Color(1.0, 1.0, 1.0, 1.0));
	self.resizeCtrl:SetBackColorBlendMode(Turbine.UI.BlendMode.Multiply);
	self.resizeCtrl:SetBackground(resizerIcon);
	self.resizeCtrl:SetZOrder(zOrder);
	
	self.Minimums = 
	{
		Width = minimumWidth;
		Height = minimumHeight
	};
	self:SetResizerPosition(); 

	self.resizeCtrl.MouseDown = function(sender,args)
	  sender.dragStartX = args.X;
	  sender.dragStartY = args.Y;
	  sender.dragging = true;
	end
		
	self.resizeCtrl.MouseUp = function(sender,args)
	  sender.dragging = false;
	end
	
	self.resizeCtrl.MouseMove = function(sender,args)
		local width, height = self:GetSize();
		
		if (sender.dragging) then
			self:SetSize(width + (args.X - sender.dragStartX), 	height + (args.Y - sender.dragStartY));
			if (self:GetWidth() < self.Minimums.Width) then
				self:SetWidth(self.Minimums.Width);
			end
			if (self:GetHeight() < self.Minimums.Height) then
				self:SetHeight(self.Minimums.Height);
			end
			local displayWidth, displayHeight = Turbine.UI.Display.GetSize();
			if (self:GetLeft() + self:GetWidth() > displayWidth) then
				self:SetWidth(self:GetWidth() - (self:GetLeft() + self:GetWidth() - displayWidth));
			end
			if (self:GetTop() + self:GetHeight() > displayHeight) then
				self:SetHeight(self:GetHeight() - (self:GetTop() + self:GetHeight() - displayHeight));
			end
			self:LayoutControls();
		end
		
		self:SetResizerPosition(); 
	end
end

function Window_SetResizerPosition(self, resizerWidth, resizerHeight, offsetWidth, offsetHeight)
	self.resizeCtrl:SetPosition(self:GetWidth() - resizerWidth - offsetWidth, self:GetHeight() - resizerHeight - offsetHeight); 
end

function Window_UpdateResizer(self, minimumWidth, minimumHeight)
	self.Minimums.Width = minimumWidth;
	self.Minimums.Height = minimumHeight;
	
	local adjustLayout = false;
	
	if minimumWidth > self:GetWidth() then
		self:SetWidth(minimumWidth);
		adjustLayout = true;
	end
	if minimumHeight > self:GetHeight() then
		self:SetHeight(minimumHeight);
		adjustLayout = true;
	end
	if adjustLayout then
		self:LayoutControls();
		self:SetResizerPosition(); 
		self:CheckWindowPosition();
	end
	
	return adjustLayout;
end

function Window_CheckWindowPosition(self)
	local displayWidth, displayHeight = Turbine.UI.Display.GetSize();
	
	if self:GetLeft() + self:GetWidth() > displayWidth then
		self:SetLeft(displayWidth - self:GetWidth());
	end
	
	if self:GetLeft() < 0 then
		self:SetLeft(0);
	end
	
	if self:GetTop() + self:GetHeight() > displayHeight then
		self:SetTop(displayHeight - self:GetHeight());
	end
	
	if self:GetTop() < 0 then
		self:SetTop(0);
	end
end

function Window_SetIconImage(shortcutImages, icon, imageType, iconSet)
	iconSet = iconSet or Globals.Icons;
	
	if shortcutImages[imageType] == nil then
		icon:SetBackground(iconSet[imageType]);
		shortcutImages[imageType] = icon:GetBackground();
	else
		icon:SetBackground(shortcutImages[imageType]);
	end
end