-- Version 1.1

WordWrapper = class(Turbine.UI.Control);

function WordWrapper:Constructor(parent)
	Turbine.UI.Control.Constructor(self);

	self:SetParent(parent);
	
	self.DefaultWidth = 1000;
	self.MinimumWidth = 8;
	
	self.WidthLabel = Turbine.UI.Label();
	self.WidthLabel:SetParent(self);
	self.WidthLabel:SetMultiline(false);
	self.WidthLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	self.WidthLabel:SetFont(Globals.Fonts.Label);
	self.WidthLabel:SetPosition(0, -100);
	self.WidthLabel:SetHeight(50);
	
	self.WidthScrollBar = Turbine.UI.Lotro.ScrollBar();
	self.WidthScrollBar:SetParent(self.WidthLabel);
	self.WidthScrollBar:SetOrientation(Turbine.UI.Orientation.Horizontal);
	self.WidthScrollBar:SetPosition(0, 0);
	self.WidthScrollBar:SetSize(100, 10);
	self.WidthScrollBar:SetZOrder(self.WidthLabel:GetZOrder() + 1);
	
	self.WidthLabel:SetHorizontalScrollBar(self.WidthScrollBar);

	self.HeightLabel = Turbine.UI.Label();
	self.HeightLabel:SetParent(self);
	self.HeightLabel:SetMultiline(true);
	self.HeightLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	self.HeightLabel:SetFont(Globals.Fonts.Label);
	self.HeightLabel:SetPosition(0, -200);
	self.HeightLabel:SetHeight(50);

	self.HeightScrollBar = Turbine.UI.Lotro.ScrollBar();
	self.HeightScrollBar:SetParent(self.HeightLabel);
	self.HeightScrollBar:SetOrientation(Turbine.UI.Orientation.Vertical);
	self.HeightScrollBar:SetPosition(0, 0);
	self.HeightScrollBar:SetSize(10, 100);
	self.HeightScrollBar:SetZOrder(self.HeightLabel:GetZOrder() + 1);
	
	self.HeightLabel:SetVerticalScrollBar(self.HeightScrollBar);
end

function WordWrapper:SetFont(font)
	self.WidthLabel:SetFont(font);
	self.HeightLabel:SetFont(font);
	self.WidthLabel:SetText(self.WidthLabel:GetText());
end

function WordWrapper:SetMarkupEnabled(markupEnabled)
	self.WidthLabel:SetMarkupEnabled(markupEnabled);
end

function WordWrapper:GetTextLength(text, defaultWidth)
	self.WidthLabel:SetText(text);
	
	if defaultWidth == nil then
		defaultWidth = self.DefaultWidth;
	end
	
	local leftWidth, rightWidth = 0, defaultWidth;
	self.WidthLabel:SetWidth(rightWidth);
	while self.WidthScrollBar:IsVisible() do
		leftWidth = rightWidth;
		rightWidth = rightWidth + defaultWidth;
		self.WidthLabel:SetWidth(rightWidth);
	end
	
	while rightWidth > leftWidth + 1 and rightWidth > self.MinimumWidth do
		local testWidth = leftWidth + math.floor((rightWidth - leftWidth) / 2);
		self.WidthLabel:SetWidth(testWidth); 
		--Turbine.Shell.WriteLine(tostring(leftWidth) .. ", " .. tostring(rightWidth) .. " = " .. tostring(testWidth) .. " - " .. tostring(self.WidthScrollBar:IsVisible()));
		if self.WidthScrollBar:IsVisible() then
			leftWidth = testWidth;
		else
			rightWidth = testWidth;
		end
	end

	if rightWidth < self.MinimumWidth then
		rightWidth = self.MinimumWidth;
	end
	
	return rightWidth;
end

function WordWrapper:SetHeight(label, labelWidth, textWidth, lineHeight, padding)
	local rows = math.ceil(textWidth / labelWidth);
	if rows == 0 then
		rows = 1;
	end
	local height = rows * lineHeight;
	
	self.HeightLabel:SetTop(0 - height * 2);
	self.HeightLabel:SetSize(labelWidth, height);
	self.HeightLabel:SetText(label:GetText());

	while self.HeightScrollBar:IsVisible() do
		height = height + lineHeight / 2.0;
		self.HeightLabel:SetHeight(height);
	end
	label:SetSize(labelWidth, height + (padding or 1));
end
